@@ -16,12 +16,13 @@ namespace sf {
       SEXP& geometry_cols,
       SEXP& polygon_id,
       SEXP& linestring_id,
+      std::string xyzm,
       bool close = true
   ) {
 
     //Rcpp::Rcout << "dont keep " << std::endl;
 
-    Rcpp::List sfc = sfheaders::sfc::sfc_polygon( x, geometry_cols, polygon_id, linestring_id, close );
+    Rcpp::List sfc = sfheaders::sfc::sfc_polygon( x, geometry_cols, polygon_id, linestring_id, xyzm, close );
     SEXP ids = sfheaders::utils::get_ids( x, polygon_id );
 
     //return sfc;
@@ -37,6 +38,7 @@ namespace sf {
       Rcpp::String& id_column,
       SEXP& multiline_ids,
       Rcpp::String& line_ids,
+      std::string xyzm,
       bool close = true
   ) {
     Rcpp::IntegerMatrix polygon_positions = sfheaders::utils::id_positions( multiline_ids );
@@ -44,9 +46,21 @@ namespace sf {
     Rcpp::StringVector df_names = df.names();
     Rcpp::IntegerVector property_idx = sfheaders::utils::where_is( property_cols, df_names );
 
-    Rcpp::List sfc = sfheaders::sfc::sfc_polygon( df, geometry_cols, line_ids, polygon_positions, close );
+    Rcpp::List sfc = sfheaders::sfc::sfc_polygon( df, geometry_cols, line_ids, polygon_positions, xyzm, close );
 
-    return sfheaders::sf::create_sf( df, sfc, id_column, property_cols, property_idx, row_idx );
+    Rcpp::List res = Rcpp::List::create(
+      Rcpp::_["df"] = df,
+      Rcpp::_["sfc"] = sfc,
+      Rcpp::_["id_column"] = id_column,
+      Rcpp::_["property_cols"] = property_cols,
+      Rcpp::_["property_idx"] = property_idx,
+      Rcpp::_["row_idx"] = row_idx,
+      Rcpp::_["line_positions"] = polygon_positions
+    );
+
+    return res;
+
+    //return sfheaders::sf::create_sf( df, sfc, id_column, property_cols, property_idx, row_idx );
   }
 
   inline SEXP sf_polygon(
@@ -56,6 +70,7 @@ namespace sf {
       int& id_column,
       SEXP& multiline_ids,
       int& linestring_id,
+      std::string xyzm,
       bool close = true
   ) {
 
@@ -66,7 +81,7 @@ namespace sf {
     Rcpp::String str_linestring_id = df_names[ linestring_id ];
     Rcpp::String str_id_column = df_names[ id_column ];
 
-    return sf_polygon( df, str_geometry_cols, str_property_cols, str_id_column, multiline_ids, str_linestring_id, close );
+    return sf_polygon( df, str_geometry_cols, str_property_cols, str_id_column, multiline_ids, str_linestring_id, xyzm, close );
   }
 
   inline SEXP sf_polygon(
@@ -75,13 +90,14 @@ namespace sf {
       Rcpp::StringVector& property_cols,
       Rcpp::String& polygon_id,
       Rcpp::String& linestring_id,
+      std::string xyzm,
       bool close = true
   ) {
 
     // Rcpp::Rcout << "sf_polygon" << std::endl;
     Rcpp::StringVector df_names = df.names();
     SEXP multiline_ids = df[ polygon_id ];
-    return sf_polygon( df, geometry_cols, property_cols, polygon_id, multiline_ids, linestring_id, close );
+    return sf_polygon( df, geometry_cols, property_cols, polygon_id, multiline_ids, linestring_id, xyzm, close );
   }
 
   inline SEXP sf_polygon(
@@ -90,6 +106,7 @@ namespace sf {
       Rcpp::IntegerVector& property_cols,
       int& polygon_id,
       int& linestring_id,
+      std::string xyzm,
       bool close = true
   ) {
 
@@ -98,7 +115,7 @@ namespace sf {
     Rcpp::StringVector str_property_cols = df_names[ property_cols ];
     Rcpp::String id_column = df_names[ polygon_id ];
     Rcpp::String line_id = df_names[ linestring_id ];
-    return sf_polygon( df, str_geometry_cols, str_property_cols, id_column, line_id, close );
+    return sf_polygon( df, str_geometry_cols, str_property_cols, id_column, line_id, xyzm, close );
   }
 
   inline SEXP sf_polygon(
@@ -107,12 +124,13 @@ namespace sf {
       Rcpp::IntegerVector& property_cols,
       int& polygon_id,
       int& linestring_id,
+      std::string xyzm,
       bool close = true
   ) {
 
     sfheaders::utils::column_exists( im, polygon_id );
     Rcpp::DataFrame df = Rcpp::as< Rcpp::DataFrame >( im );
-    return sf_polygon( df, geometry_cols, property_cols, polygon_id, linestring_id, close );
+    return sf_polygon( df, geometry_cols, property_cols, polygon_id, linestring_id, xyzm, close );
   }
 
   inline SEXP sf_polygon(
@@ -121,10 +139,11 @@ namespace sf {
       Rcpp::IntegerVector& property_cols,
       int& polygon_id,
       int& linestring_id,
+      std::string xyzm,
       bool close = true
   ) {
     Rcpp::DataFrame df = Rcpp::as< Rcpp::DataFrame >( nm );
-    return sf_polygon( df, geometry_cols, property_cols, polygon_id, linestring_id, close );
+    return sf_polygon( df, geometry_cols, property_cols, polygon_id, linestring_id, xyzm, close );
   }
 
   inline SEXP sf_polygon(
@@ -133,11 +152,12 @@ namespace sf {
       Rcpp::StringVector& property_cols,
       Rcpp::String& polygon_id,
       Rcpp::String& linestring_id,
+      std::string xyzm,
       bool close = true
   ) {
     Rcpp::DataFrame df = Rcpp::as< Rcpp::DataFrame >( im );
     SEXP multiline_ids = df[ polygon_id ];
-    return sf_polygon( df, geometry_cols, property_cols, polygon_id, multiline_ids, linestring_id, close );
+    return sf_polygon( df, geometry_cols, property_cols, polygon_id, multiline_ids, linestring_id, xyzm, close );
   }
 
 
@@ -147,11 +167,12 @@ namespace sf {
       Rcpp::StringVector& property_cols,
       Rcpp::String& polygon_id,
       Rcpp::String& linestring_id,
+      std::string xyzm,
       bool close = true
   ) {
     Rcpp::DataFrame df = Rcpp::as< Rcpp::DataFrame >( nm );
     SEXP multiline_ids = df[ polygon_id ];
-    return sf_polygon( df, geometry_cols, property_cols, polygon_id, multiline_ids, linestring_id, close );
+    return sf_polygon( df, geometry_cols, property_cols, polygon_id, multiline_ids, linestring_id, xyzm, close );
   }
 
 
@@ -166,6 +187,7 @@ namespace sf {
       Rcpp::IntegerVector& property_cols,
       int& polygon_id,
       int& linestring_id,
+      std::string xyzm,
       bool close = true
   ) {
     switch( TYPEOF( x ) ) {
@@ -173,20 +195,20 @@ namespace sf {
       if( Rf_isMatrix( x ) ) {
       SEXP xc = Rcpp::clone( x );
       Rcpp::IntegerMatrix im = Rcpp::as< Rcpp::IntegerMatrix >( xc );
-      return sf_polygon( im, geometry_cols, property_cols, polygon_id, linestring_id, close );
+      return sf_polygon( im, geometry_cols, property_cols, polygon_id, linestring_id, xyzm, close );
     }
     }
     case REALSXP: {
       if( Rf_isMatrix( x ) ) {
       SEXP xc = Rcpp::clone( x );
       Rcpp::NumericMatrix nm = Rcpp::as< Rcpp::NumericMatrix >( xc );
-      return sf_polygon( nm, geometry_cols, property_cols, polygon_id, linestring_id, close );
+      return sf_polygon( nm, geometry_cols, property_cols, polygon_id, linestring_id, xyzm, close );
     }
     }
     case VECSXP: {
       if( Rf_inherits( x, "data.frame" ) ) {
       Rcpp::DataFrame df = Rcpp::as< Rcpp::DataFrame >( x );
-      return sf_polygon( df, geometry_cols, property_cols, polygon_id, linestring_id, close );
+      return sf_polygon( df, geometry_cols, property_cols, polygon_id, linestring_id, xyzm, close );
     }
     }
     default: {
@@ -203,6 +225,7 @@ namespace sf {
       Rcpp::StringVector& property_cols,
       Rcpp::String& polygon_id,
       Rcpp::String& linestring_id,
+      std::string xyzm,
       bool close = true
   ) {
 
@@ -211,20 +234,20 @@ namespace sf {
       if( Rf_isMatrix( x ) ) {
       SEXP xc = Rcpp::clone( x );
       Rcpp::IntegerMatrix im = Rcpp::as< Rcpp::IntegerMatrix >( xc );
-      return sf_polygon( im, geometry_cols, property_cols, polygon_id, linestring_id, close );
+      return sf_polygon( im, geometry_cols, property_cols, polygon_id, linestring_id, xyzm, close );
     }
     }
     case REALSXP: {
       if( Rf_isMatrix( x ) ) {
       SEXP xc = Rcpp::clone( x );
       Rcpp::NumericMatrix nm = Rcpp::as< Rcpp::NumericMatrix >( xc );
-      return sf_polygon( nm, geometry_cols, property_cols, polygon_id, linestring_id, close );
+      return sf_polygon( nm, geometry_cols, property_cols, polygon_id, linestring_id, xyzm, close );
     }
     }
     case VECSXP: {
       if( Rf_inherits( x, "data.frame" ) ) {
       Rcpp::DataFrame df = Rcpp::as< Rcpp::DataFrame >( x );
-      return sf_polygon( df, geometry_cols, property_cols, polygon_id, linestring_id, close );
+      return sf_polygon( df, geometry_cols, property_cols, polygon_id, linestring_id, xyzm, close );
     }
     }
     default: {
@@ -240,12 +263,13 @@ namespace sf {
       SEXP& geometry_cols,
       SEXP& polygon_id,
       SEXP& linestring_id,
+      std::string xyzm,
       bool& close,
       bool& keep
   ) {
 
     if( !keep ) {
-      return sf_polygon( x, geometry_cols, polygon_id, linestring_id, close );
+      return sf_polygon( x, geometry_cols, polygon_id, linestring_id, xyzm, close );
     }
 
 
@@ -255,21 +279,49 @@ namespace sf {
 
     if( !Rf_isNull( polygon_id ) && Rf_isNull( linestring_id ) ) {
       SEXP linestring_id2 = polygon_id;
-      return sf_polygon( x, geometry_cols, polygon_id, linestring_id2, close, keep );
+      return sf_polygon( x, geometry_cols, polygon_id, linestring_id2, xyzm, close, keep );
     }
 
     if( Rf_isNull( polygon_id ) && !Rf_isNull( linestring_id ) ) {
       // the entire object is a polygon
-      Rcpp::List sfc = sfheaders::sfc::sfc_polygon( x, geometry_cols, polygon_id, linestring_id, close );
+      Rcpp::List sfc = sfheaders::sfc::sfc_polygon( x, geometry_cols, polygon_id, linestring_id, xyzm, close );
       SEXP property_columns = sfheaders::utils::other_columns( x, geometry_cols, linestring_id );
-      return sfheaders::sf::create_sf( x, sfc, property_columns );
+
+      Rcpp::IntegerVector property_idx = sfheaders::utils::where_is( property_columns, x );
+      Rcpp::IntegerMatrix line_positions(1,2);
+      line_positions(0,0) = 0;
+      line_positions(0,1) = sfheaders::utils::sexp_n_row( x ) - 1;
+
+      Rcpp::List res = Rcpp::List::create(
+        Rcpp::_["x"] = x,
+        Rcpp::_["sfc"] = sfc,
+        Rcpp::_["property_cols"] = property_columns,
+        Rcpp::_["property_idx"] = property_idx,
+        Rcpp::_["line_positions"] = line_positions
+      );
+
+      return res;
     }
 
     if( Rf_isNull( polygon_id ) && Rf_isNull( linestring_id ) ) {
       // the entire object is a polygon
-      Rcpp::List sfc = sfheaders::sfc::sfc_polygon( x, geometry_cols, polygon_id, linestring_id, close );
+      Rcpp::List sfc = sfheaders::sfc::sfc_polygon( x, geometry_cols, polygon_id, linestring_id, xyzm, close );
       SEXP property_columns = sfheaders::utils::other_columns( x, geometry_cols );
-      return sfheaders::sf::create_sf( x, sfc, property_columns );
+
+      Rcpp::IntegerVector property_idx = sfheaders::utils::where_is( property_columns, x );
+      Rcpp::IntegerMatrix line_positions(1,2);
+      line_positions(0,0) = 0;
+      line_positions(0,1) = sfheaders::utils::sexp_n_row( x ) - 1;
+
+      Rcpp::List res = Rcpp::List::create(
+        Rcpp::_["x"] = x,
+        Rcpp::_["sfc"] = sfc,
+        Rcpp::_["property_cols"] = property_columns,
+        Rcpp::_["property_idx"] = property_idx,
+        Rcpp::_["line_positions"] = line_positions
+      );
+
+      return res;
     }
 
     if( !Rf_isNull( polygon_id ) && !Rf_isNull( linestring_id ) ) {
@@ -289,7 +341,7 @@ namespace sf {
 
         int i_polygon_id_col = iv_polygon_id_col[0];
         int i_linestring_id_col = iv_linestring_id_col[0];
-        return sf_polygon( x, iv_geometry_cols, iv_property_cols, i_polygon_id_col, i_linestring_id_col, close );
+        return sf_polygon( x, iv_geometry_cols, iv_property_cols, i_polygon_id_col, i_linestring_id_col, xyzm, close );
 
       }
       case STRSXP: {
@@ -303,7 +355,7 @@ namespace sf {
 
         Rcpp::String s_polygon_id_col = sv_polygon_id_col[0];
         Rcpp::String s_linestring_id_col = sv_linestring_id_col[0];
-        return sf_polygon( x, sv_geometry_cols, sv_property_cols, s_polygon_id_col, s_linestring_id_col, close );
+        return sf_polygon( x, sv_geometry_cols, sv_property_cols, s_polygon_id_col, s_linestring_id_col, xyzm, close );
       }
       default: {
         Rcpp::stop("sfheaders - unsupported polygon type");  // #nocov
@@ -315,7 +367,6 @@ namespace sf {
     return Rcpp::List::create(); // ??
   }
 
-
 } // sf
 } // sfheaders
 
