@@ -26,21 +26,20 @@ namespace sfc {
 
   // #nocov start
   inline SEXP sfc_multilinestring(
-      Rcpp::IntegerMatrix& im
+      Rcpp::IntegerMatrix& im,
+      std::string xyzm
   ) {
     //
     Rcpp::NumericVector bbox = sfheaders::bbox::start_bbox();
     Rcpp::NumericVector z_range = sfheaders::zm::start_z_range();
     Rcpp::NumericVector m_range = sfheaders::zm::start_m_range();
 
-    R_xlen_t n_col = im.ncol();
-
     Rcpp::List sfc( 1 );
 
     sfheaders::bbox::calculate_bbox( bbox, im );
-    sfheaders::zm::calculate_zm_ranges( n_col, z_range, m_range, im );
+    sfheaders::zm::calculate_zm_ranges( z_range, m_range, im, xyzm );
 
-    Rcpp::List mp = sfheaders::sfg::sfg_multilinestring( im );
+    Rcpp::List mp = sfheaders::sfg::sfg_multilinestring( im, xyzm );
 
     sfc[0] = mp;
     sfheaders::sfc::make_sfc( sfc, sfheaders::sfc::SFC_MULTILINESTRING, bbox, z_range, m_range );
@@ -49,21 +48,20 @@ namespace sfc {
 
 
   inline SEXP sfc_multilinestring(
-      Rcpp::NumericMatrix& nm
+      Rcpp::NumericMatrix& nm,
+      std::string xyzm
   ) {
     //
     Rcpp::NumericVector bbox = sfheaders::bbox::start_bbox();
     Rcpp::NumericVector z_range = sfheaders::zm::start_z_range();
     Rcpp::NumericVector m_range = sfheaders::zm::start_m_range();
 
-    R_xlen_t n_col = nm.ncol();
-
     Rcpp::List sfc( 1 );
 
     sfheaders::bbox::calculate_bbox( bbox, nm );
-    sfheaders::zm::calculate_zm_ranges( n_col, z_range, m_range, nm );
+    sfheaders::zm::calculate_zm_ranges( z_range, m_range, nm, xyzm );
 
-    Rcpp::List mp = sfheaders::sfg::sfg_multilinestring( nm );
+    Rcpp::List mp = sfheaders::sfg::sfg_multilinestring( nm, xyzm );
 
     sfc[0] = mp;
     sfheaders::sfc::make_sfc( sfc, sfheaders::sfc::SFC_MULTILINESTRING, bbox, z_range, m_range );
@@ -72,21 +70,20 @@ namespace sfc {
   }
 
   inline SEXP sfc_multilinestring(
-      Rcpp::DataFrame& df
+      Rcpp::DataFrame& df,
+      std::string xyzm
   ) {
 
     Rcpp::NumericVector bbox = sfheaders::bbox::start_bbox();
     Rcpp::NumericVector z_range = sfheaders::zm::start_z_range();
     Rcpp::NumericVector m_range = sfheaders::zm::start_m_range();
 
-    R_xlen_t n_col = df.ncol();
-
-    Rcpp::List sfc( 1 );
+        Rcpp::List sfc( 1 );
 
     sfheaders::bbox::calculate_bbox( bbox, df );
-    sfheaders::zm::calculate_zm_ranges( n_col, z_range, m_range, df );
+    sfheaders::zm::calculate_zm_ranges( z_range, m_range, df, xyzm );
 
-    Rcpp::List mp = sfheaders::sfg::sfg_multilinestring( df );
+    Rcpp::List mp = sfheaders::sfg::sfg_multilinestring( df, xyzm );
 
     sfc[0] = mp;
     sfheaders::sfc::make_sfc( sfc, sfheaders::sfc::SFC_MULTILINESTRING, bbox, z_range, m_range );
@@ -95,21 +92,23 @@ namespace sfc {
 
 
   inline SEXP sfc_multilinestring(
-      Rcpp::IntegerVector& iv
+      Rcpp::IntegerVector& iv,
+      std::string xyzm
   ) {
     int n_col = iv.size();
     Rcpp::IntegerMatrix im(1, n_col);
     im(0, Rcpp::_ ) = iv;
-    return sfc_multilinestring( im );
+    return sfc_multilinestring( im, xyzm );
   }
 
   inline SEXP sfc_multilinestring(
-      Rcpp::NumericVector& nv
+      Rcpp::NumericVector& nv,
+      std::string xyzm
   ) {
     int n_col = nv.size();
     Rcpp::NumericMatrix nm(1, n_col);
     nm(0, Rcpp::_ ) = nv;
-    return sfc_multilinestring( nm );
+    return sfc_multilinestring( nm, xyzm );
   }
 
 // inline SEXP sfc_multilinestring(
@@ -198,7 +197,8 @@ namespace sfc {
 
   // no subsetting to do; so just turn the object into a matrix;
   inline SEXP sfc_multilinestring(
-      SEXP& x
+      SEXP& x,
+      std::string xyzm
   ) {
     // here there is no subsetting required;
     // we have to assume the columsn are in order x,y,(z,(m))
@@ -206,25 +206,25 @@ namespace sfc {
     case INTSXP: {
       if( Rf_isMatrix( x ) ) {
       Rcpp::IntegerMatrix im = Rcpp::as< Rcpp::IntegerMatrix >( x );
-      return sfc_multilinestring( im );
+      return sfc_multilinestring( im, xyzm );
     } else {
       Rcpp::IntegerVector iv = Rcpp::as< Rcpp::IntegerVector >( x );
-      return sfc_multilinestring( iv );
+      return sfc_multilinestring( iv, xyzm );
     }
     }
     case REALSXP: {
       if( Rf_isMatrix( x ) ) {
       Rcpp::NumericMatrix nm = Rcpp::as< Rcpp::NumericMatrix >( x );
-      return sfc_multilinestring( nm );
+      return sfc_multilinestring( nm, xyzm );
     } else {
       Rcpp::NumericVector nv = Rcpp::as< Rcpp::NumericVector >( x );
-      return sfc_multilinestring( nv );
+      return sfc_multilinestring( nv, xyzm );
     }
     }
     case VECSXP: {
       if( Rf_inherits( x, "data.frame" ) ) {
       Rcpp::DataFrame df = Rcpp::as< Rcpp::DataFrame >( x );
-      return sfc_multilinestring( df );
+      return sfc_multilinestring( df, xyzm );
     }
     }
     default: {
@@ -236,7 +236,8 @@ namespace sfc {
 
   inline SEXP sfc_multilinestring(
       Rcpp::IntegerMatrix& im,
-      Rcpp::IntegerVector& geometry_cols
+      Rcpp::IntegerVector& geometry_cols,
+      std::string xyzm
   ) {
     Rcpp::NumericVector bbox = sfheaders::bbox::start_bbox();
     Rcpp::NumericVector z_range = sfheaders::zm::start_z_range();
@@ -244,11 +245,10 @@ namespace sfc {
 
     sfheaders::bbox::calculate_bbox( bbox, im, geometry_cols );
 
-    R_xlen_t n_col = im.ncol();
-    sfheaders::zm::calculate_zm_ranges( n_col, z_range, m_range, im, geometry_cols );
+    sfheaders::zm::calculate_zm_ranges( z_range, m_range, im, geometry_cols, xyzm );
 
     Rcpp::List sfc( 1 );
-    Rcpp::List sfg = sfheaders::sfg::sfg_multilinestring( im, geometry_cols );
+    Rcpp::List sfg = sfheaders::sfg::sfg_multilinestring( im, geometry_cols, xyzm );
     sfc[0] = sfg;
     sfheaders::sfc::make_sfc( sfc, sfheaders::sfc::SFC_MULTILINESTRING, bbox, z_range, m_range );
     return sfc;
@@ -256,7 +256,8 @@ namespace sfc {
 
   inline SEXP sfc_multilinestring(
       Rcpp::IntegerMatrix& im,
-      Rcpp::StringVector& geometry_cols
+      Rcpp::StringVector& geometry_cols,
+      std::string xyzm
   ) {
 
     Rcpp::DataFrame df = Rcpp::as< Rcpp::DataFrame >( im );
@@ -264,13 +265,11 @@ namespace sfc {
     Rcpp::NumericVector z_range = sfheaders::zm::start_z_range();
     Rcpp::NumericVector m_range = sfheaders::zm::start_m_range();
 
-    R_xlen_t n_col = df.ncol();
-
     sfheaders::bbox::calculate_bbox( bbox, df, geometry_cols );
-    sfheaders::zm::calculate_zm_ranges( n_col, z_range, m_range, df, geometry_cols );
+    sfheaders::zm::calculate_zm_ranges( z_range, m_range, df, geometry_cols, xyzm );
 
     Rcpp::List sfc( 1 );
-    Rcpp::List sfg = sfheaders::sfg::sfg_multilinestring( df, geometry_cols );
+    Rcpp::List sfg = sfheaders::sfg::sfg_multilinestring( df, geometry_cols, xyzm );
     sfc[0] = sfg;
     sfheaders::sfc::make_sfc( sfc, sfheaders::sfc::SFC_MULTILINESTRING, bbox, z_range, m_range );
     return sfc;
@@ -278,19 +277,18 @@ namespace sfc {
 
   inline SEXP sfc_multilinestring(
       Rcpp::NumericMatrix& nm,
-      Rcpp::IntegerVector& geometry_cols
+      Rcpp::IntegerVector& geometry_cols,
+      std::string xyzm
   ) {
     Rcpp::NumericVector bbox = sfheaders::bbox::start_bbox();
     Rcpp::NumericVector z_range = sfheaders::zm::start_z_range();
     Rcpp::NumericVector m_range = sfheaders::zm::start_m_range();
 
-    R_xlen_t n_col = nm.ncol();
-
     sfheaders::bbox::calculate_bbox( bbox, nm, geometry_cols );
-    sfheaders::zm::calculate_zm_ranges( n_col, z_range, m_range, nm, geometry_cols );
+    sfheaders::zm::calculate_zm_ranges( z_range, m_range, nm, geometry_cols, xyzm );
 
     Rcpp::List sfc( 1 );
-    Rcpp::List sfg = sfheaders::sfg::sfg_multilinestring( nm, geometry_cols );
+    Rcpp::List sfg = sfheaders::sfg::sfg_multilinestring( nm, geometry_cols, xyzm );
     sfc[0] = sfg;
     sfheaders::sfc::make_sfc( sfc, sfheaders::sfc::SFC_MULTILINESTRING, bbox, z_range, m_range );
     return sfc;
@@ -298,7 +296,8 @@ namespace sfc {
 
   inline SEXP sfc_multilinestring(
       Rcpp::NumericMatrix& nm,
-      Rcpp::StringVector& geometry_cols
+      Rcpp::StringVector& geometry_cols,
+      std::string xyzm
   ) {
 
     Rcpp::DataFrame df = Rcpp::as< Rcpp::DataFrame >( nm );
@@ -306,13 +305,11 @@ namespace sfc {
     Rcpp::NumericVector z_range = sfheaders::zm::start_z_range();
     Rcpp::NumericVector m_range = sfheaders::zm::start_m_range();
 
-    R_xlen_t n_col = df.ncol();
-
     sfheaders::bbox::calculate_bbox( bbox, df, geometry_cols );
-    sfheaders::zm::calculate_zm_ranges( n_col, z_range, m_range, df, geometry_cols );
+    sfheaders::zm::calculate_zm_ranges( z_range, m_range, df, geometry_cols, xyzm );
 
     Rcpp::List sfc( 1 );
-    Rcpp::List sfg = sfheaders::sfg::sfg_multilinestring( df, geometry_cols );
+    Rcpp::List sfg = sfheaders::sfg::sfg_multilinestring( df, geometry_cols, xyzm );
     sfc[0] = sfg;
     sfheaders::sfc::make_sfc( sfc, sfheaders::sfc::SFC_MULTILINESTRING, bbox, z_range, m_range );
     return sfc;
@@ -320,7 +317,8 @@ namespace sfc {
 
   inline SEXP sfc_multilinestring(
       Rcpp::DataFrame& df,
-      Rcpp::IntegerVector& geometry_cols
+      Rcpp::IntegerVector& geometry_cols,
+      std::string xyzm
   ) {
     Rcpp::NumericVector bbox = sfheaders::bbox::start_bbox();
     Rcpp::NumericVector z_range = sfheaders::zm::start_z_range();
@@ -328,11 +326,10 @@ namespace sfc {
 
     sfheaders::bbox::calculate_bbox( bbox, df, geometry_cols );
 
-    R_xlen_t n_col = df.ncol();
-    sfheaders::zm::calculate_zm_ranges( n_col, z_range, m_range, df, geometry_cols );
+    sfheaders::zm::calculate_zm_ranges( z_range, m_range, df, geometry_cols, xyzm );
 
     Rcpp::List sfc( 1 );
-    Rcpp::List sfg = sfheaders::sfg::sfg_multilinestring( df, geometry_cols );
+    Rcpp::List sfg = sfheaders::sfg::sfg_multilinestring( df, geometry_cols, xyzm );
     sfc[0] = sfg;
     sfheaders::sfc::make_sfc( sfc, sfheaders::sfc::SFC_MULTILINESTRING, bbox, z_range, m_range );
     return sfc;
@@ -340,7 +337,8 @@ namespace sfc {
 
   inline SEXP sfc_multilinestring(
       Rcpp::DataFrame& df,
-      Rcpp::StringVector& geometry_cols
+      Rcpp::StringVector& geometry_cols,
+      std::string xyzm
   ) {
 
     Rcpp::NumericVector bbox = sfheaders::bbox::start_bbox();
@@ -349,11 +347,10 @@ namespace sfc {
 
     sfheaders::bbox::calculate_bbox( bbox, df, geometry_cols );
 
-    R_xlen_t n_col = df.ncol();
-    sfheaders::zm::calculate_zm_ranges( n_col, z_range, m_range, df, geometry_cols );
+    sfheaders::zm::calculate_zm_ranges( z_range, m_range, df, geometry_cols, xyzm );
 
     Rcpp::List sfc( 1 );
-    Rcpp::List sfg = sfheaders::sfg::sfg_multilinestring( df, geometry_cols );
+    Rcpp::List sfg = sfheaders::sfg::sfg_multilinestring( df, geometry_cols, xyzm );
     sfc[0] = sfg;
     sfheaders::sfc::make_sfc( sfc, sfheaders::sfc::SFC_MULTILINESTRING, bbox, z_range, m_range );
     return sfc;
@@ -361,13 +358,14 @@ namespace sfc {
 
   inline SEXP sfc_multilinestring(
       SEXP& x,
-      Rcpp::StringVector& geometry_cols
+      Rcpp::StringVector& geometry_cols,
+      std::string xyzm
   ) {
     switch( TYPEOF( x ) ) {
     case INTSXP: {
       if( Rf_isMatrix( x ) ) {
       Rcpp::IntegerMatrix im = Rcpp::as< Rcpp::IntegerMatrix >( x );
-      return sfc_multilinestring( im, geometry_cols );
+      return sfc_multilinestring( im, geometry_cols, xyzm );
       // } else {
       //   Rcpp::IntegerVector iv = Rcpp::as< Rcpp::IntegerVector >( x );
       //   return sfc_multilinestring( iv, geometry_cols );
@@ -376,7 +374,7 @@ namespace sfc {
     case REALSXP: {
       if( Rf_isMatrix( x ) ) {
       Rcpp::NumericMatrix nm = Rcpp::as< Rcpp::NumericMatrix >( x );
-      return sfc_multilinestring( nm, geometry_cols );
+      return sfc_multilinestring( nm, geometry_cols, xyzm );
       // } else {
       //   Rcpp::NumericVector nv = Rcpp::as< Rcpp::NumericVector >( x );
       //   return sfc_multilinestring( nv, geometry_cols );
@@ -385,7 +383,7 @@ namespace sfc {
     case VECSXP: {
       if( Rf_inherits( x, "data.frame" ) ) {
       Rcpp::DataFrame df = Rcpp::as< Rcpp::DataFrame >( x );
-      return sfc_multilinestring( df, geometry_cols );
+      return sfc_multilinestring( df, geometry_cols, xyzm );
     }
     }
     default: {
@@ -398,13 +396,14 @@ namespace sfc {
 
   inline SEXP sfc_multilinestring(
       SEXP& x,
-      Rcpp::IntegerVector& geometry_cols
+      Rcpp::IntegerVector& geometry_cols,
+      std::string xyzm
   ) {
     switch( TYPEOF( x ) ) {
     case INTSXP: {
       if( Rf_isMatrix( x ) ) {
       Rcpp::IntegerMatrix im = Rcpp::as< Rcpp::IntegerMatrix >( x );
-      return sfc_multilinestring( im, geometry_cols );
+      return sfc_multilinestring( im, geometry_cols, xyzm );
       // } else {
       //   Rcpp::IntegerVector iv = Rcpp::as< Rcpp::IntegerVector >( x );
       //   return sfc_multilinestring( iv, geometry_cols );
@@ -413,7 +412,7 @@ namespace sfc {
     case REALSXP: {
       if( Rf_isMatrix( x ) ) {
       Rcpp::NumericMatrix nm = Rcpp::as< Rcpp::NumericMatrix >( x );
-      return sfc_multilinestring( nm, geometry_cols );
+      return sfc_multilinestring( nm, geometry_cols, xyzm );
       // } else {
       //   Rcpp::NumericVector nv = Rcpp::as< Rcpp::NumericVector >( x );
       //   return sfc_multilinestring( nv, geometry_cols );
@@ -422,7 +421,7 @@ namespace sfc {
     case VECSXP: {
       if( Rf_inherits( x, "data.frame" ) ) {
       Rcpp::DataFrame df = Rcpp::as< Rcpp::DataFrame >( x );
-      return sfc_multilinestring( df, geometry_cols );
+      return sfc_multilinestring( df, geometry_cols, xyzm );
     }
     }
     default: {
@@ -435,7 +434,8 @@ namespace sfc {
   // no subsetting to do (except for columns)
   inline SEXP sfc_multilinestring(
       SEXP& x,
-      SEXP& geometry_cols
+      SEXP& geometry_cols,
+      std::string xyzm
   ) {
     sfheaders::utils::geometry_column_check( geometry_cols );
 
@@ -443,11 +443,11 @@ namespace sfc {
     case REALSXP: {}
     case INTSXP: {
       Rcpp::IntegerVector iv = Rcpp::as< Rcpp::IntegerVector >( geometry_cols );
-      return sfc_multilinestring( x, iv );
+      return sfc_multilinestring( x, iv, xyzm );
     }
     case STRSXP: {
       Rcpp::StringVector sv = Rcpp::as< Rcpp::StringVector >( geometry_cols );
-      return sfc_multilinestring( x, sv );
+      return sfc_multilinestring( x, sv, xyzm );
     }
     default: {
       Rcpp::stop("sfheaders - unknown column type");
@@ -473,7 +473,8 @@ namespace sfc {
     Rcpp::DataFrame& df,
     Rcpp::StringVector& geometry_cols,
     Rcpp::String& linestring_id,
-    Rcpp::IntegerMatrix& multilinestring_positions
+    Rcpp::IntegerMatrix& multilinestring_positions,
+    std::string xyzm
   ) {
     Rcpp::NumericVector bbox = sfheaders::bbox::start_bbox();
     Rcpp::NumericVector z_range = sfheaders::zm::start_z_range();
@@ -481,8 +482,7 @@ namespace sfc {
 
     sfheaders::bbox::calculate_bbox( bbox, df, geometry_cols );
 
-    R_xlen_t n_col = df.ncol();
-    sfheaders::zm::calculate_zm_ranges( n_col, z_range, m_range, df, geometry_cols );
+    sfheaders::zm::calculate_zm_ranges( z_range, m_range, df, geometry_cols, xyzm );
 
     R_xlen_t n_multilinestrings = multilinestring_positions.nrow();
 
@@ -501,7 +501,7 @@ namespace sfc {
       start = multilinestring_positions( i, 0 );
       end = multilinestring_positions( i, 1 );
       Rcpp::DataFrame df_subset = sfheaders::utils::subset_dataframe( df_keep, df_names, start, end );
-      sfc( i ) = sfheaders::sfg::sfg_multilinestring( df_subset, geometry_cols, linestring_id );
+      sfc( i ) = sfheaders::sfg::sfg_multilinestring( df_subset, geometry_cols, linestring_id, xyzm );
     }
 
     sfheaders::sfc::make_sfc( sfc, sfheaders::sfc::SFC_MULTILINESTRING, bbox, z_range, m_range );
@@ -512,41 +512,45 @@ namespace sfc {
       Rcpp::DataFrame& df,
       Rcpp::IntegerVector& geometry_cols,
       int& linestring_id,
-      Rcpp::IntegerMatrix& multilinestring_positions
+      Rcpp::IntegerMatrix& multilinestring_positions,
+      std::string xyzm
   ) {
 
     // get the string-values of the data.frame names
     Rcpp::StringVector df_names = df.names();
     Rcpp::StringVector str_geometry_cols = df_names[ geometry_cols ];
     Rcpp::String str_linestring_id = df_names[ linestring_id ];
-    return sfc_multilinestring( df, str_geometry_cols, str_linestring_id, multilinestring_positions );
+    return sfc_multilinestring( df, str_geometry_cols, str_linestring_id, multilinestring_positions, xyzm );
   }
 
   inline SEXP sfc_multilinestring(
       Rcpp::DataFrame& df,
       Rcpp::IntegerVector& geometry_cols,
       SEXP& multilinestring_ids,
-      int& linestring_id
+      int& linestring_id,
+      std::string xyzm
   ) {
     Rcpp::IntegerMatrix multilinestring_positions = sfheaders::utils::id_positions( multilinestring_ids );
-    return sfc_multilinestring( df, geometry_cols, linestring_id, multilinestring_positions );
+    return sfc_multilinestring( df, geometry_cols, linestring_id, multilinestring_positions, xyzm );
   }
 
   inline SEXP sfc_multilinestring(
       Rcpp::DataFrame& df,
       Rcpp::StringVector& geometry_cols,
       SEXP& multilinestring_ids,  // can be int, double, string, ...
-      Rcpp::String& linestring_id
+      Rcpp::String& linestring_id,
+      std::string xyzm
   ) {
     Rcpp::IntegerMatrix multilinestring_positions = sfheaders::utils::id_positions( multilinestring_ids );
-    return sfc_multilinestring(df, geometry_cols, linestring_id, multilinestring_positions );
+    return sfc_multilinestring(df, geometry_cols, linestring_id, multilinestring_positions, xyzm );
   }
 
   inline SEXP sfc_multilinestring(
     Rcpp::IntegerMatrix& im,
     Rcpp::IntegerVector& geometry_cols,
     int& linestring_id,
-    Rcpp::IntegerMatrix& multilinestring_positions
+    Rcpp::IntegerMatrix& multilinestring_positions,
+    std::string xyzm
   ) {
     Rcpp::NumericVector bbox = sfheaders::bbox::start_bbox();
     Rcpp::NumericVector z_range = sfheaders::zm::start_z_range();
@@ -554,8 +558,7 @@ namespace sfc {
 
     sfheaders::bbox::calculate_bbox( bbox, im, geometry_cols );
 
-    R_xlen_t n_col = im.ncol();
-    sfheaders::zm::calculate_zm_ranges( n_col, z_range, m_range, im, geometry_cols );
+    sfheaders::zm::calculate_zm_ranges( z_range, m_range, im, geometry_cols, xyzm );
 
 
     R_xlen_t n_multilinestrings = multilinestring_positions.nrow();
@@ -569,7 +572,7 @@ namespace sfc {
       start = multilinestring_positions( i, 0 );
       end = multilinestring_positions( i, 1 );
       Rcpp::IntegerMatrix im2 = im( Rcpp::Range(start, end), Rcpp::_ );
-      sfc( i ) = sfheaders::sfg::sfg_multilinestring( im2, geometry_cols, linestring_id );
+      sfc( i ) = sfheaders::sfg::sfg_multilinestring( im2, geometry_cols, linestring_id, xyzm );
     }
 
     sfheaders::sfc::make_sfc( sfc, sfheaders::sfc::SFC_MULTILINESTRING, bbox, z_range, m_range );
@@ -580,17 +583,19 @@ namespace sfc {
       Rcpp::IntegerMatrix& im,
       Rcpp::IntegerVector& geometry_cols,
       Rcpp::IntegerVector& multilinestring_ids,
-      int& linestring_id
+      int& linestring_id,
+      std::string xyzm
   ) {
     Rcpp::IntegerMatrix multilinestring_positions = sfheaders::utils::id_positions( multilinestring_ids );
-    return sfc_multilinestring( im, geometry_cols, linestring_id, multilinestring_positions );
+    return sfc_multilinestring( im, geometry_cols, linestring_id, multilinestring_positions, xyzm );
   }
 
   inline SEXP sfc_multilinestring(
     Rcpp::NumericMatrix& nm,
     Rcpp::IntegerVector& geometry_cols,
     int& linestring_id,
-    Rcpp::IntegerMatrix& multilinestring_positions
+    Rcpp::IntegerMatrix& multilinestring_positions,
+    std::string xyzm
   ) {
     Rcpp::NumericVector bbox = sfheaders::bbox::start_bbox();
     Rcpp::NumericVector z_range = sfheaders::zm::start_z_range();
@@ -598,8 +603,7 @@ namespace sfc {
 
     sfheaders::bbox::calculate_bbox( bbox, nm, geometry_cols );
 
-    R_xlen_t n_col = nm.ncol();
-    sfheaders::zm::calculate_zm_ranges( n_col, z_range, m_range, nm, geometry_cols );
+    sfheaders::zm::calculate_zm_ranges( z_range, m_range, nm, geometry_cols, xyzm );
 
 
     R_xlen_t n_multilinestrings = multilinestring_positions.nrow();
@@ -613,7 +617,7 @@ namespace sfc {
       start = multilinestring_positions( i, 0 );
       end = multilinestring_positions( i, 1 );
       Rcpp::NumericMatrix nm2 = nm( Rcpp::Range(start, end), Rcpp::_ );
-      sfc( i ) = sfheaders::sfg::sfg_multilinestring( nm2, geometry_cols, linestring_id );
+      sfc( i ) = sfheaders::sfg::sfg_multilinestring( nm2, geometry_cols, linestring_id, xyzm );
     }
 
     sfheaders::sfc::make_sfc( sfc, sfheaders::sfc::SFC_MULTILINESTRING, bbox, z_range, m_range );
@@ -624,86 +628,94 @@ namespace sfc {
       Rcpp::NumericMatrix& nm,
       Rcpp::IntegerVector& geometry_cols,
       Rcpp::NumericVector& multilinestring_ids,
-      int& linestring_id
+      int& linestring_id,
+      std::string xyzm
   ) {
     Rcpp::IntegerMatrix multilinestring_positions = sfheaders::utils::id_positions( multilinestring_ids );
-    return sfc_multilinestring( nm, geometry_cols, linestring_id, multilinestring_positions );
+    return sfc_multilinestring( nm, geometry_cols, linestring_id, multilinestring_positions, xyzm );
   }
 
   inline SEXP sfc_multilinestring(
       Rcpp::IntegerMatrix& im,
       Rcpp::IntegerVector& geometry_cols,
       int& multilinestring_id,
-      int& linestring_id
+      int& linestring_id,
+      std::string xyzm
   ) {
     sfheaders::utils::column_exists( im, multilinestring_id );
     Rcpp::IntegerVector multilinestring_ids = im( Rcpp::_, multilinestring_id );
-    return sfc_multilinestring( im, geometry_cols, multilinestring_ids, linestring_id );
+    return sfc_multilinestring( im, geometry_cols, multilinestring_ids, linestring_id, xyzm );
   }
 
   inline SEXP sfc_multilinestring(
       Rcpp::NumericMatrix& nm,
       Rcpp::IntegerVector& geometry_cols,
       int& multilinestring_id,
-      int& linestring_id
+      int& linestring_id,
+      std::string xyzm
   ) {
     sfheaders::utils::column_exists( nm, multilinestring_id );
     Rcpp::NumericVector multilinestring_ids = nm( Rcpp::_, multilinestring_id );
-    return sfc_multilinestring( nm, geometry_cols, multilinestring_ids, linestring_id );
+    return sfc_multilinestring( nm, geometry_cols, multilinestring_ids, linestring_id, xyzm );
   }
 
   inline SEXP sfc_multilinestring(
       Rcpp::DataFrame& df,
       Rcpp::StringVector& geometry_cols,
       Rcpp::String& multilinestring_id,
-      Rcpp::String& linestring_id
+      Rcpp::String& linestring_id,
+      std::string xyzm
   ) {
     SEXP multilinestring_ids = df[ multilinestring_id ];
-    return sfc_multilinestring( df, geometry_cols, multilinestring_ids, linestring_id );
+    return sfc_multilinestring( df, geometry_cols, multilinestring_ids, linestring_id, xyzm );
   }
 
   inline SEXP sfc_multilinestring(
       Rcpp::DataFrame& df,
       Rcpp::IntegerVector& geometry_cols,
       int& multilinestring_id,
-      int& linestring_id
+      int& linestring_id,
+      std::string xyzm
   ) {
     sfheaders::utils::column_exists( df, multilinestring_id );
     SEXP multilinestring_ids = df[ multilinestring_id ];
-    return sfc_multilinestring( df, geometry_cols, multilinestring_ids, linestring_id );
+    return sfc_multilinestring( df, geometry_cols, multilinestring_ids, linestring_id, xyzm );
   }
 
   inline SEXP sfc_multilinestring(
       Rcpp::IntegerMatrix& im,
       Rcpp::StringVector& geometry_cols,
       Rcpp::String& multilinestring_id,
-      Rcpp::String& linestring_id
+      Rcpp::String& linestring_id,
+      std::string xyzm
   ) {
     Rcpp::DataFrame df = Rcpp::as< Rcpp::DataFrame >( im );
-    return sfc_multilinestring( df, geometry_cols, multilinestring_id, linestring_id );
+    return sfc_multilinestring( df, geometry_cols, multilinestring_id, linestring_id, xyzm );
   }
 
   inline SEXP sfc_multilinestring(
       Rcpp::NumericMatrix& nm,
       Rcpp::StringVector& geometry_cols,
       Rcpp::String& multilinestring_id,
-      Rcpp::String& linestring_id
+      Rcpp::String& linestring_id,
+      std::string xyzm
   ) {
     Rcpp::DataFrame df = Rcpp::as< Rcpp::DataFrame >( nm );
-    return sfc_multilinestring( df, geometry_cols, multilinestring_id, linestring_id );
+    return sfc_multilinestring( df, geometry_cols, multilinestring_id, linestring_id, xyzm );
   }
 
   inline SEXP sfc_multilinestring(
       SEXP& x,
       Rcpp::IntegerVector& geometry_cols,
       int& multilinestring_id,
-      int& linestring_id
+      int& linestring_id,
+      std::string xyzm
   ) {
     switch( TYPEOF( x ) ) {
     case INTSXP: {
       if( Rf_isMatrix( x ) ) {
       Rcpp::IntegerMatrix im = Rcpp::as< Rcpp::IntegerMatrix >( x );
-      return sfc_multilinestring( im, geometry_cols, multilinestring_id, linestring_id );
+      return sfc_multilinestring( im, geometry_cols, multilinestring_id, linestring_id, xyzm );
       // } else {
       // Rcpp::IntegerVector iv = Rcpp::as< Rcpp::IntegerVector >( x );
       // return sfc_multilinestring( iv, geometry_cols, linestring_id );
@@ -712,7 +724,7 @@ namespace sfc {
     case REALSXP: {
       if( Rf_isMatrix( x ) ) {
       Rcpp::NumericMatrix nm = Rcpp::as< Rcpp::NumericMatrix >( x );
-      return sfc_multilinestring( nm, geometry_cols, multilinestring_id, linestring_id );
+      return sfc_multilinestring( nm, geometry_cols, multilinestring_id, linestring_id, xyzm );
       // } else {
       //   Rcpp::NumericVector nv = Rcpp::as< Rcpp::NumericVector >( x );
       //   return sfc_linestring( nv, geometry_cols, linestring_id );
@@ -721,7 +733,7 @@ namespace sfc {
     case VECSXP: {
       if( Rf_inherits( x, "data.frame" ) ) {
       Rcpp::DataFrame df = Rcpp::as< Rcpp::DataFrame >( x );
-      return sfc_multilinestring( df, geometry_cols, multilinestring_id, linestring_id );
+      return sfc_multilinestring( df, geometry_cols, multilinestring_id, linestring_id, xyzm );
     }
     }
     default: {
@@ -735,13 +747,14 @@ namespace sfc {
       SEXP& x,
       Rcpp::StringVector& geometry_cols,
       Rcpp::String& multilinestring_id,
-      Rcpp::String& linestring_id
+      Rcpp::String& linestring_id,
+      std::string xyzm
   ) {
     switch( TYPEOF( x ) ) {
     case INTSXP: {
       if( Rf_isMatrix( x ) ) {
       Rcpp::IntegerMatrix im = Rcpp::as< Rcpp::IntegerMatrix >( x );
-      return sfc_multilinestring( im, geometry_cols, multilinestring_id, linestring_id );
+      return sfc_multilinestring( im, geometry_cols, multilinestring_id, linestring_id, xyzm );
       // } else {
       //   Rcpp::IntegerVector iv = Rcpp::as< Rcpp::IntegerVector >( x );
       //   return sfc_multilinestring( iv, geometry_cols, linestring_id );
@@ -750,7 +763,7 @@ namespace sfc {
     case REALSXP: {
       if( Rf_isMatrix( x ) ) {
       Rcpp::NumericMatrix nm = Rcpp::as< Rcpp::NumericMatrix >( x );
-      return sfc_multilinestring( nm, geometry_cols, multilinestring_id, linestring_id );
+      return sfc_multilinestring( nm, geometry_cols, multilinestring_id, linestring_id, xyzm );
       // } else {
       //   Rcpp::NumericVector nv = Rcpp::as< Rcpp::NumericVector >( x );
       //   return sfc_multilinestring( nv, geometry_cols, linestring_id );
@@ -759,7 +772,7 @@ namespace sfc {
     case VECSXP: {
       if( Rf_inherits( x, "data.frame" ) ) {
       Rcpp::DataFrame df = Rcpp::as< Rcpp::DataFrame >( x );
-      return sfc_multilinestring( df, geometry_cols, multilinestring_id, linestring_id );
+      return sfc_multilinestring( df, geometry_cols, multilinestring_id, linestring_id, xyzm );
     }
     }
     default: {
@@ -775,20 +788,21 @@ namespace sfc {
       SEXP& x,
       SEXP& geometry_cols,
       SEXP& multilinestring_id,
-      SEXP& linestring_id
+      SEXP& linestring_id,
+      std::string xyzm
   ) {
 
     if( Rf_isNull( geometry_cols ) ) {
       // make this all the other columns, then send back in
       SEXP geometry_cols2 = sfheaders::utils::other_columns( x, multilinestring_id, linestring_id );
-      return sfc_multilinestring( x, geometry_cols2, multilinestring_id, linestring_id );
+      return sfc_multilinestring( x, geometry_cols2, multilinestring_id, linestring_id, xyzm );
     }
 
     // From now on, geometry_cols will never be null
 
     if( Rf_isNull( linestring_id ) &&
         Rf_isNull( multilinestring_id ) ) {
-      return sfc_multilinestring( x, geometry_cols );
+      return sfc_multilinestring( x, geometry_cols, xyzm );
     }
     //
     // // JUST multilinestring_id == each multilinestring is just one line
@@ -815,24 +829,21 @@ namespace sfc {
       case INTSXP: {
         if( Rf_isMatrix( x ) ) {
         Rcpp::IntegerMatrix im = Rcpp::as< Rcpp::IntegerMatrix >( x );
-        R_xlen_t n_col = im.ncol();
-        sfheaders::zm::calculate_zm_ranges( n_col, z_range, m_range, im, geometry_cols );
+        sfheaders::zm::calculate_zm_ranges( z_range, m_range, im, geometry_cols, xyzm );
         break;
       }
       }
       case REALSXP: {
         if( Rf_isMatrix( x ) ) {
         Rcpp::NumericMatrix nm = Rcpp::as< Rcpp::NumericMatrix >( x );
-        R_xlen_t n_col = nm.ncol();
-        sfheaders::zm::calculate_zm_ranges( n_col, z_range, m_range, nm, geometry_cols );
+        sfheaders::zm::calculate_zm_ranges( z_range, m_range, nm, geometry_cols, xyzm );
         break;
       }
       }
       case VECSXP: {
         if( Rf_inherits( x, "data.frame" ) ) {
         Rcpp::DataFrame df = Rcpp::as< Rcpp::DataFrame >( x );
-        R_xlen_t n_col = df.ncol();
-        sfheaders::zm::calculate_zm_ranges( n_col, z_range, m_range, df, geometry_cols );
+        sfheaders::zm::calculate_zm_ranges( z_range, m_range, df, geometry_cols, xyzm );
         break;
       }
       }
@@ -841,7 +852,7 @@ namespace sfc {
       }
       }
 
-      sfc[0] = sfheaders::sfg::sfg_multilinestring( x, geometry_cols, linestring_id );
+      sfc[0] = sfheaders::sfg::sfg_multilinestring( x, geometry_cols, linestring_id, xyzm );
       sfheaders::sfc::make_sfc( sfc, sfheaders::sfc::SFC_MULTILINESTRING, bbox, z_range, m_range );
       return sfc;
     }
@@ -854,7 +865,7 @@ namespace sfc {
       // and given each multilinestring only has one line, can we simply set the linestring_id to be the
       // same as the multilinestring_id column; will taht work??
       SEXP linestring_id2 = multilinestring_id;
-      return sfc_multilinestring( x, geometry_cols, multilinestring_id, linestring_id2 );
+      return sfc_multilinestring( x, geometry_cols, multilinestring_id, linestring_id2, xyzm );
     }
 
     // if( !Rf_isNull( linestring_id ) &&
@@ -892,7 +903,7 @@ namespace sfc {
         Rcpp::IntegerVector iv_linestring_id_col = Rcpp::as< Rcpp::IntegerVector >( linestring_id );
         int i_multilinestring_id_col = iv_multilinestring_id_col[0];
         int i_linestring_id_col = iv_linestring_id_col[0];
-        return sfc_multilinestring( x, iv_geometry_cols, i_multilinestring_id_col, i_linestring_id_col );
+        return sfc_multilinestring( x, iv_geometry_cols, i_multilinestring_id_col, i_linestring_id_col, xyzm );
 
       }
       case STRSXP: {
@@ -901,7 +912,7 @@ namespace sfc {
         Rcpp::StringVector sv_linestring_id_col = Rcpp::as< Rcpp::StringVector >( linestring_id );
         Rcpp::String s_multilinestring_id_col = sv_multilinestring_id_col[0];
         Rcpp::String s_linestring_id_col = sv_linestring_id_col[0];
-        return sfc_multilinestring( x, sv_geometry_cols, s_multilinestring_id_col, s_linestring_id_col );
+        return sfc_multilinestring( x, sv_geometry_cols, s_multilinestring_id_col, s_linestring_id_col, xyzm );
       }
       default: {
         Rcpp::stop("sfheaders - unsupported multilinestring type");  // #nocov
