@@ -25,21 +25,20 @@ namespace sfc {
   }
 
 inline SEXP sfc_multipoint(
-    Rcpp::IntegerMatrix& im
+    Rcpp::IntegerMatrix& im,
+    std::string xyzm
 ) {
   //
   Rcpp::NumericVector bbox = sfheaders::bbox::start_bbox();
   Rcpp::NumericVector z_range = sfheaders::zm::start_z_range();
   Rcpp::NumericVector m_range = sfheaders::zm::start_m_range();
 
-  R_xlen_t n_col = im.ncol();
-
   Rcpp::List sfc( 1 );
 
   sfheaders::bbox::calculate_bbox( bbox, im );
-  sfheaders::zm::calculate_zm_ranges( n_col, z_range, m_range, im );
+  sfheaders::zm::calculate_zm_ranges( z_range, m_range, im, xyzm );
 
-  Rcpp::IntegerMatrix mp = sfheaders::sfg::sfg_multipoint( im );
+  Rcpp::IntegerMatrix mp = sfheaders::sfg::sfg_multipoint( im, xyzm );
 
   sfc[0] = mp;
   sfheaders::sfc::make_sfc( sfc, sfheaders::sfc::SFC_MULTIPOINT, bbox, z_range, m_range );
@@ -48,21 +47,20 @@ inline SEXP sfc_multipoint(
 
 
 inline SEXP sfc_multipoint(
-    Rcpp::NumericMatrix& nm
+    Rcpp::NumericMatrix& nm,
+    std::string xyzm
 ) {
 
   Rcpp::NumericVector bbox = sfheaders::bbox::start_bbox();
   Rcpp::NumericVector z_range = sfheaders::zm::start_z_range();
   Rcpp::NumericVector m_range = sfheaders::zm::start_m_range();
 
-  R_xlen_t n_col = nm.ncol();
-
   Rcpp::List sfc( 1 );
 
   sfheaders::bbox::calculate_bbox( bbox, nm );
-  sfheaders::zm::calculate_zm_ranges( n_col, z_range, m_range, nm );
+  sfheaders::zm::calculate_zm_ranges( z_range, m_range, nm, xyzm );
 
-  Rcpp::NumericMatrix mp = sfheaders::sfg::sfg_multipoint( nm );
+  Rcpp::NumericMatrix mp = sfheaders::sfg::sfg_multipoint( nm, xyzm );
 
   sfc[0] = mp;
   sfheaders::sfc::make_sfc( sfc, sfheaders::sfc::SFC_MULTIPOINT, bbox, z_range, m_range );
@@ -70,19 +68,18 @@ inline SEXP sfc_multipoint(
 }
 
 inline SEXP sfc_multipoint(
-    Rcpp::DataFrame& df
+    Rcpp::DataFrame& df,
+    std::string xyzm
 ) {
 
   Rcpp::NumericVector bbox = sfheaders::bbox::start_bbox();
   Rcpp::NumericVector z_range = sfheaders::zm::start_z_range();
   Rcpp::NumericVector m_range = sfheaders::zm::start_m_range();
 
-  R_xlen_t n_col = df.ncol();
-
   sfheaders::bbox::calculate_bbox( bbox, df );
-  sfheaders::zm::calculate_zm_ranges( n_col, z_range, m_range, df );
+  sfheaders::zm::calculate_zm_ranges( z_range, m_range, df, xyzm );
 
-  Rcpp::NumericMatrix mp = sfheaders::sfg::sfg_multipoint( df );
+  Rcpp::NumericMatrix mp = sfheaders::sfg::sfg_multipoint( df, xyzm );
 
   Rcpp::List sfc( 1 );
   sfc[0] = mp;
@@ -92,21 +89,23 @@ inline SEXP sfc_multipoint(
 
 
 inline SEXP sfc_multipoint(
-    Rcpp::IntegerVector& iv
+    Rcpp::IntegerVector& iv,
+    std::string xyzm
 ) {
   int n_col = iv.size();
   Rcpp::IntegerMatrix im(1, n_col);
   im(0, Rcpp::_ ) = iv;
-  return sfc_multipoint( im );
+  return sfc_multipoint( im, xyzm );
 }
 
 inline SEXP sfc_multipoint(
-    Rcpp::NumericVector& nv
+    Rcpp::NumericVector& nv,
+    std::string xyzm
 ) {
   int n_col = nv.size();
   Rcpp::NumericMatrix nm(1, n_col);
   nm(0, Rcpp::_ ) = nv;
-  return sfc_multipoint( nm );
+  return sfc_multipoint( nm, xyzm );
 }
 
 // inline SEXP sfc_multipoint(
@@ -175,7 +174,8 @@ inline SEXP sfc_multipoint(
 
 // no subsetting to do; so just turn the object into a matrix;
 inline SEXP sfc_multipoint(
-    SEXP& x
+    SEXP& x,
+    std::string xyzm
 ) {
   // here there is no subsetting required;
   // we have to assume the columsn are in order x,y,(z,(m))
@@ -185,26 +185,26 @@ inline SEXP sfc_multipoint(
     if( Rf_isMatrix( x ) ) {
     SEXP xc = Rcpp::clone( x );
     Rcpp::IntegerMatrix im = Rcpp::as< Rcpp::IntegerMatrix >( xc );
-    return sfc_multipoint( im );
+    return sfc_multipoint( im, xyzm );
   } else {
     Rcpp::IntegerVector iv = Rcpp::as< Rcpp::IntegerVector >( x );
-    return sfc_multipoint( iv );
+    return sfc_multipoint( iv, xyzm );
   }
   }
   case REALSXP: {
     if( Rf_isMatrix( x ) ) {
     SEXP xc = Rcpp::clone( x );
     Rcpp::NumericMatrix nm = Rcpp::as< Rcpp::NumericMatrix >( xc );
-    return sfc_multipoint( nm );
+    return sfc_multipoint( nm, xyzm );
   } else {
     Rcpp::NumericVector nv = Rcpp::as< Rcpp::NumericVector >( x );
-    return sfc_multipoint( nv );
+    return sfc_multipoint( nv, xyzm );
   }
   }
   case VECSXP: {
     if( Rf_inherits( x, "data.frame" ) ) {
     Rcpp::DataFrame df = Rcpp::as< Rcpp::DataFrame >( x );
-    return sfc_multipoint( df );
+    return sfc_multipoint( df, xyzm );
   }
   }
   default: {
@@ -216,7 +216,8 @@ inline SEXP sfc_multipoint(
 
 inline SEXP sfc_multipoint(
     Rcpp::IntegerMatrix& im,
-    Rcpp::IntegerVector& geometry_cols
+    Rcpp::IntegerVector& geometry_cols,
+    std::string xyzm
 ) {
   Rcpp::NumericVector bbox = sfheaders::bbox::start_bbox();
   Rcpp::NumericVector z_range = sfheaders::zm::start_z_range();
@@ -224,11 +225,10 @@ inline SEXP sfc_multipoint(
 
   sfheaders::bbox::calculate_bbox( bbox, im, geometry_cols );
 
-  R_xlen_t n_col = im.ncol();
-  sfheaders::zm::calculate_zm_ranges( n_col, z_range, m_range, im, geometry_cols );
+  sfheaders::zm::calculate_zm_ranges( z_range, m_range, im, geometry_cols, xyzm );
 
   Rcpp::List sfc( 1 );
-  Rcpp::NumericMatrix sfg = sfheaders::sfg::sfg_multipoint( im, geometry_cols );
+  Rcpp::NumericMatrix sfg = sfheaders::sfg::sfg_multipoint( im, geometry_cols, xyzm );
   sfc[0] = sfg;
   sfheaders::sfc::make_sfc( sfc, sfheaders::sfc::SFC_MULTIPOINT, bbox, z_range, m_range );
   return sfc;
@@ -236,7 +236,8 @@ inline SEXP sfc_multipoint(
 
 inline SEXP sfc_multipoint(
     Rcpp::IntegerMatrix& im,
-    Rcpp::StringVector& geometry_cols
+    Rcpp::StringVector& geometry_cols,
+    std::string xyzm
 ) {
 
   Rcpp::DataFrame df = Rcpp::as< Rcpp::DataFrame >( im );
@@ -246,11 +247,10 @@ inline SEXP sfc_multipoint(
 
   sfheaders::bbox::calculate_bbox( bbox, df, geometry_cols );
 
-  R_xlen_t n_col = df.ncol();
-  sfheaders::zm::calculate_zm_ranges( n_col, z_range, m_range, df, geometry_cols );
+  sfheaders::zm::calculate_zm_ranges( z_range, m_range, df, geometry_cols, xyzm );
 
   Rcpp::List sfc( 1 );
-  Rcpp::NumericMatrix sfg = sfheaders::sfg::sfg_multipoint( df, geometry_cols );
+  Rcpp::NumericMatrix sfg = sfheaders::sfg::sfg_multipoint( df, geometry_cols, xyzm );
   sfc[0] = sfg;
   sfheaders::sfc::make_sfc( sfc, sfheaders::sfc::SFC_MULTIPOINT, bbox, z_range, m_range );
   return sfc;
@@ -258,7 +258,8 @@ inline SEXP sfc_multipoint(
 
 inline SEXP sfc_multipoint(
     Rcpp::NumericMatrix& nm,
-    Rcpp::IntegerVector& geometry_cols
+    Rcpp::IntegerVector& geometry_cols,
+    std::string xyzm
 ) {
   Rcpp::NumericVector bbox = sfheaders::bbox::start_bbox();
   Rcpp::NumericVector z_range = sfheaders::zm::start_z_range();
@@ -266,11 +267,10 @@ inline SEXP sfc_multipoint(
 
   sfheaders::bbox::calculate_bbox( bbox, nm, geometry_cols );
 
-  R_xlen_t n_col = nm.ncol();
-  sfheaders::zm::calculate_zm_ranges( n_col, z_range, m_range, nm, geometry_cols );
+  sfheaders::zm::calculate_zm_ranges( z_range, m_range, nm, geometry_cols, xyzm );
 
   Rcpp::List sfc( 1 );
-  Rcpp::NumericMatrix sfg = sfheaders::sfg::sfg_multipoint( nm, geometry_cols );
+  Rcpp::NumericMatrix sfg = sfheaders::sfg::sfg_multipoint( nm, geometry_cols, xyzm );
   sfc[0] = sfg;
   sfheaders::sfc::make_sfc( sfc, sfheaders::sfc::SFC_MULTIPOINT, bbox, z_range, m_range );
   return sfc;
@@ -278,7 +278,8 @@ inline SEXP sfc_multipoint(
 
 inline SEXP sfc_multipoint(
     Rcpp::NumericMatrix& nm,
-    Rcpp::StringVector& geometry_cols
+    Rcpp::StringVector& geometry_cols,
+    std::string xyzm
 ) {
 
   Rcpp::DataFrame df = Rcpp::as< Rcpp::DataFrame >( nm );
@@ -286,13 +287,11 @@ inline SEXP sfc_multipoint(
   Rcpp::NumericVector z_range = sfheaders::zm::start_z_range();
   Rcpp::NumericVector m_range = sfheaders::zm::start_m_range();
 
-  R_xlen_t n_col = df.ncol();
-
   sfheaders::bbox::calculate_bbox( bbox, df, geometry_cols );
-  sfheaders::zm::calculate_zm_ranges( n_col, z_range, m_range, df, geometry_cols );
+  sfheaders::zm::calculate_zm_ranges( z_range, m_range, df, geometry_cols, xyzm );
 
   Rcpp::List sfc( 1 );
-  Rcpp::NumericMatrix sfg = sfheaders::sfg::sfg_multipoint( df, geometry_cols );
+  Rcpp::NumericMatrix sfg = sfheaders::sfg::sfg_multipoint( df, geometry_cols, xyzm );
   sfc[0] = sfg;
   sfheaders::sfc::make_sfc( sfc, sfheaders::sfc::SFC_MULTIPOINT, bbox, z_range, m_range );
   return sfc;
@@ -300,7 +299,8 @@ inline SEXP sfc_multipoint(
 
 inline SEXP sfc_multipoint(
     Rcpp::DataFrame& df,
-    Rcpp::IntegerVector& geometry_cols
+    Rcpp::IntegerVector& geometry_cols,
+    std::string xyzm
 ) {
   Rcpp::NumericVector bbox = sfheaders::bbox::start_bbox();
   Rcpp::NumericVector z_range = sfheaders::zm::start_z_range();
@@ -308,11 +308,10 @@ inline SEXP sfc_multipoint(
 
   sfheaders::bbox::calculate_bbox( bbox, df, geometry_cols );
 
-  R_xlen_t n_col = df.ncol();
-  sfheaders::zm::calculate_zm_ranges( n_col, z_range, m_range, df, geometry_cols );
+  sfheaders::zm::calculate_zm_ranges( z_range, m_range, df, geometry_cols, xyzm );
 
   Rcpp::List sfc( 1 );
-  Rcpp::NumericMatrix sfg = sfheaders::sfg::sfg_multipoint( df, geometry_cols );
+  Rcpp::NumericMatrix sfg = sfheaders::sfg::sfg_multipoint( df, geometry_cols, xyzm );
   sfc[0] = sfg;
   sfheaders::sfc::make_sfc( sfc, sfheaders::sfc::SFC_MULTIPOINT, bbox, z_range, m_range );
   return sfc;
@@ -320,7 +319,8 @@ inline SEXP sfc_multipoint(
 
 inline SEXP sfc_multipoint(
     Rcpp::DataFrame& df,
-    Rcpp::StringVector& geometry_cols
+    Rcpp::StringVector& geometry_cols,
+    std::string xyzm
 ) {
 
   Rcpp::NumericVector bbox = sfheaders::bbox::start_bbox();
@@ -329,11 +329,10 @@ inline SEXP sfc_multipoint(
 
   sfheaders::bbox::calculate_bbox( bbox, df, geometry_cols );
 
-  R_xlen_t n_col = df.ncol();
-  sfheaders::zm::calculate_zm_ranges( n_col, z_range, m_range, df, geometry_cols );
+  sfheaders::zm::calculate_zm_ranges( z_range, m_range, df, geometry_cols, xyzm );
 
   Rcpp::List sfc( 1 );
-  Rcpp::NumericMatrix sfg = sfheaders::sfg::sfg_multipoint( df, geometry_cols );
+  Rcpp::NumericMatrix sfg = sfheaders::sfg::sfg_multipoint( df, geometry_cols, xyzm );
   sfc[0] = sfg;
   sfheaders::sfc::make_sfc( sfc, sfheaders::sfc::SFC_MULTIPOINT, bbox, z_range, m_range );
   return sfc;
@@ -341,14 +340,15 @@ inline SEXP sfc_multipoint(
 
 inline SEXP sfc_multipoint(
     SEXP& x,
-    Rcpp::StringVector& geometry_cols
+    Rcpp::StringVector& geometry_cols,
+    std::string xyzm
 ) {
   switch( TYPEOF( x ) ) {
   case INTSXP: {
     if( Rf_isMatrix( x ) ) {
     SEXP xc = Rcpp::clone( x );
     Rcpp::IntegerMatrix im = Rcpp::as< Rcpp::IntegerMatrix >( xc );
-    return sfc_multipoint( im, geometry_cols );
+    return sfc_multipoint( im, geometry_cols, xyzm );
     // } else {
     //   Rcpp::IntegerVector iv = Rcpp::as< Rcpp::IntegerVector >( x );
     //   return sfc_multipoint( iv, geometry_cols );
@@ -358,7 +358,7 @@ inline SEXP sfc_multipoint(
     if( Rf_isMatrix( x ) ) {
     SEXP xc = Rcpp::clone( x );
     Rcpp::NumericMatrix nm = Rcpp::as< Rcpp::NumericMatrix >( xc );
-    return sfc_multipoint( nm, geometry_cols );
+    return sfc_multipoint( nm, geometry_cols, xyzm );
     // } else {
     //   Rcpp::NumericVector nv = Rcpp::as< Rcpp::NumericVector >( x );
     //   return sfc_multipoint( nv, geometry_cols );
@@ -367,7 +367,7 @@ inline SEXP sfc_multipoint(
   case VECSXP: {
     if( Rf_inherits( x, "data.frame" ) ) {
     Rcpp::DataFrame df = Rcpp::as< Rcpp::DataFrame >( x );
-    return sfc_multipoint( df, geometry_cols );
+    return sfc_multipoint( df, geometry_cols, xyzm );
   }
   }
   default: {
@@ -379,14 +379,15 @@ inline SEXP sfc_multipoint(
 
 inline SEXP sfc_multipoint(
     SEXP& x,
-    Rcpp::IntegerVector& geometry_cols
+    Rcpp::IntegerVector& geometry_cols,
+    std::string xyzm
 ) {
   switch( TYPEOF( x ) ) {
   case INTSXP: {
     if( Rf_isMatrix( x ) ) {
     SEXP xc = Rcpp::clone( x );
     Rcpp::IntegerMatrix im = Rcpp::as< Rcpp::IntegerMatrix >( xc );
-    return sfc_multipoint( im, geometry_cols );
+    return sfc_multipoint( im, geometry_cols, xyzm );
     // } else {
     //   Rcpp::IntegerVector iv = Rcpp::as< Rcpp::IntegerVector >( x );
     //   return sfc_multipoint( iv, geometry_cols );
@@ -396,7 +397,7 @@ inline SEXP sfc_multipoint(
     if( Rf_isMatrix( x ) ) {
     SEXP xc = Rcpp::clone( x );
     Rcpp::NumericMatrix nm = Rcpp::as< Rcpp::NumericMatrix >( xc );
-    return sfc_multipoint( nm, geometry_cols );
+    return sfc_multipoint( nm, geometry_cols, xyzm );
     // } else {
     //   Rcpp::NumericVector nv = Rcpp::as< Rcpp::NumericVector >( x );
     //   return sfc_multipoint( nv, geometry_cols );
@@ -405,7 +406,7 @@ inline SEXP sfc_multipoint(
   case VECSXP: {
     if( Rf_inherits( x, "data.frame" ) ) {
     Rcpp::DataFrame df = Rcpp::as< Rcpp::DataFrame >( x );
-    return sfc_multipoint( df, geometry_cols );
+    return sfc_multipoint( df, geometry_cols, xyzm );
   }
   }
   default: {
@@ -418,7 +419,8 @@ inline SEXP sfc_multipoint(
 // no subsetting to do (except for columns)
 inline SEXP sfc_multipoint(
     SEXP& x,
-    SEXP& geometry_cols
+    SEXP& geometry_cols,
+    std::string xyzm
 ) {
   sfheaders::utils::geometry_column_check( geometry_cols );
 
@@ -426,11 +428,11 @@ inline SEXP sfc_multipoint(
   case REALSXP: {}
   case INTSXP: {
     Rcpp::IntegerVector iv = Rcpp::as< Rcpp::IntegerVector >( geometry_cols );
-    return sfc_multipoint( x, iv );
+    return sfc_multipoint( x, iv, xyzm );
   }
   case STRSXP: {
     Rcpp::StringVector sv = Rcpp::as< Rcpp::StringVector >( geometry_cols );
-    return sfc_multipoint( x, sv );
+    return sfc_multipoint( x, sv, xyzm );
   }
   default: {
     Rcpp::stop("sfheaders - unknown column type");    // #nocov
@@ -442,7 +444,8 @@ inline SEXP sfc_multipoint(
 inline SEXP sfc_multipoint(
   Rcpp::IntegerMatrix& im,
   Rcpp::IntegerVector& geometry_cols,
-  Rcpp::IntegerMatrix& line_positions
+  Rcpp::IntegerMatrix& line_positions,
+  std::string xyzm
 ) {
   Rcpp::NumericVector bbox = sfheaders::bbox::start_bbox();
   Rcpp::NumericVector z_range = sfheaders::zm::start_z_range();
@@ -450,8 +453,7 @@ inline SEXP sfc_multipoint(
 
   sfheaders::bbox::calculate_bbox( bbox, im, geometry_cols );
 
-  R_xlen_t n_col = im.ncol();
-  sfheaders::zm::calculate_zm_ranges( n_col, z_range, m_range, im, geometry_cols );
+  sfheaders::zm::calculate_zm_ranges( z_range, m_range, im, geometry_cols, xyzm );
 
 
   R_xlen_t n_lines = line_positions.nrow();
@@ -461,7 +463,7 @@ inline SEXP sfc_multipoint(
   int start;
   int end;
   if( n_lines == 1 ) {
-    sfc( 0 ) = sfheaders::sfg::sfg_multipoint( im, geometry_cols );    // #nocov
+    sfc( 0 ) = sfheaders::sfg::sfg_multipoint( im, geometry_cols, xyzm );    // #nocov
   } else {
 
     R_xlen_t i;
@@ -471,7 +473,7 @@ inline SEXP sfc_multipoint(
       end = line_positions(i, 1);
 
       Rcpp::NumericMatrix m = sfheaders::shapes::get_listMat( im, geometry_cols, start, end );  // returns a matrix
-      sfc( i ) = sfheaders::sfg::sfg_multipoint( m );
+      sfc( i ) = sfheaders::sfg::sfg_multipoint( m, xyzm );
     }
   }
 
@@ -482,16 +484,18 @@ inline SEXP sfc_multipoint(
 inline SEXP sfc_multipoint(
     Rcpp::IntegerMatrix& im,
     Rcpp::IntegerVector& geometry_cols,
-    Rcpp::IntegerVector& line_ids
+    Rcpp::IntegerVector& line_ids,
+    std::string xyzm
 ) {
   Rcpp::IntegerMatrix line_positions = sfheaders::utils::id_positions( line_ids );
-  return sfc_multipoint( im, geometry_cols, line_positions );
+  return sfc_multipoint( im, geometry_cols, line_positions, xyzm );
 }
 
 inline SEXP sfc_multipoint(
   Rcpp::NumericMatrix& nm,
   Rcpp::IntegerVector& geometry_cols,
-  Rcpp::IntegerMatrix& line_positions
+  Rcpp::IntegerMatrix& line_positions,
+  std::string xyzm
 ) {
   Rcpp::NumericVector bbox = sfheaders::bbox::start_bbox();
   Rcpp::NumericVector z_range = sfheaders::zm::start_z_range();
@@ -499,9 +503,7 @@ inline SEXP sfc_multipoint(
 
   sfheaders::bbox::calculate_bbox( bbox, nm, geometry_cols );
 
-  R_xlen_t n_col = nm.ncol();
-  sfheaders::zm::calculate_zm_ranges( n_col, z_range, m_range, nm, geometry_cols );
-
+  sfheaders::zm::calculate_zm_ranges( z_range, m_range, nm, geometry_cols, xyzm );
 
   R_xlen_t n_lines = line_positions.nrow();
 
@@ -510,7 +512,7 @@ inline SEXP sfc_multipoint(
   int start;
   int end;
   if( n_lines == 1 ) {
-    sfc( 0 ) = sfheaders::sfg::sfg_multipoint( nm, geometry_cols );  // #nocov
+    sfc( 0 ) = sfheaders::sfg::sfg_multipoint( nm, geometry_cols, xyzm );  // #nocov
   } else {
 
     R_xlen_t i;
@@ -520,7 +522,7 @@ inline SEXP sfc_multipoint(
       end = line_positions(i, 1);
 
       Rcpp::NumericMatrix m = sfheaders::shapes::get_listMat( nm, geometry_cols, start, end );  // returns a matrix
-      sfc( i ) = sfheaders::sfg::sfg_multipoint( m );
+      sfc( i ) = sfheaders::sfg::sfg_multipoint( m, xyzm );
     }
   }
 
@@ -531,16 +533,18 @@ inline SEXP sfc_multipoint(
 inline SEXP sfc_multipoint(
     Rcpp::NumericMatrix& nm,
     Rcpp::IntegerVector& geometry_cols,
-    Rcpp::NumericVector& line_ids
+    Rcpp::NumericVector& line_ids,
+    std::string xyzm
 ) {
   Rcpp::IntegerMatrix line_positions = sfheaders::utils::id_positions( line_ids );
-  return sfc_multipoint( nm, geometry_cols, line_positions );
+  return sfc_multipoint( nm, geometry_cols, line_positions, xyzm );
 }
 
 inline SEXP sfc_multipoint(
   Rcpp::DataFrame& df,
   Rcpp::StringVector& geometry_cols,
-  Rcpp::IntegerMatrix& line_positions
+  Rcpp::IntegerMatrix& line_positions,
+  std::string xyzm
 ) {
   Rcpp::NumericVector bbox = sfheaders::bbox::start_bbox();
   Rcpp::NumericVector z_range = sfheaders::zm::start_z_range();
@@ -548,8 +552,7 @@ inline SEXP sfc_multipoint(
 
   sfheaders::bbox::calculate_bbox( bbox, df, geometry_cols );
 
-  R_xlen_t n_col = df.ncol();
-  sfheaders::zm::calculate_zm_ranges( n_col, z_range, m_range, df, geometry_cols );
+  sfheaders::zm::calculate_zm_ranges( z_range, m_range, df, geometry_cols, xyzm );
 
 
   R_xlen_t n_lines = line_positions.nrow();
@@ -559,7 +562,7 @@ inline SEXP sfc_multipoint(
   int start;
   int end;
   if( n_lines == 1 ) {
-    sfc( 0 ) = sfheaders::sfg::sfg_multipoint( df, geometry_cols );  // #nocov
+    sfc( 0 ) = sfheaders::sfg::sfg_multipoint( df, geometry_cols, xyzm );  // #nocov
   } else {
 
     R_xlen_t i;
@@ -569,7 +572,7 @@ inline SEXP sfc_multipoint(
       end = line_positions(i, 1);
 
       Rcpp::NumericMatrix m = sfheaders::shapes::get_listMat( df, geometry_cols, start, end );  // returns a matrix
-      sfc( i ) = sfheaders::sfg::sfg_multipoint( m );
+      sfc( i ) = sfheaders::sfg::sfg_multipoint( m, xyzm );
     }
   }
 
@@ -580,16 +583,18 @@ inline SEXP sfc_multipoint(
 inline SEXP sfc_multipoint(
     Rcpp::DataFrame& df,
     Rcpp::StringVector& geometry_cols,
-    SEXP& line_ids
+    SEXP& line_ids,
+    std::string xyzm
 ) {
   Rcpp::IntegerMatrix line_positions = sfheaders::utils::id_positions( line_ids );
-  return sfc_multipoint( df, geometry_cols, line_positions );
+  return sfc_multipoint( df, geometry_cols, line_positions, xyzm );
 }
 
 inline SEXP sfc_multipoint(
   Rcpp::DataFrame& df,
   Rcpp::IntegerVector& geometry_cols,
-  Rcpp::IntegerMatrix& line_positions
+  Rcpp::IntegerMatrix& line_positions,
+  std::string xyzm
 ) {
   Rcpp::NumericVector bbox = sfheaders::bbox::start_bbox();
   Rcpp::NumericVector z_range = sfheaders::zm::start_z_range();
@@ -597,8 +602,7 @@ inline SEXP sfc_multipoint(
 
   sfheaders::bbox::calculate_bbox( bbox, df, geometry_cols );
 
-  R_xlen_t n_col = df.ncol();
-  sfheaders::zm::calculate_zm_ranges( n_col, z_range, m_range, df, geometry_cols );
+  sfheaders::zm::calculate_zm_ranges( z_range, m_range, df, geometry_cols, xyzm );
 
 
   R_xlen_t n_lines = line_positions.nrow();
@@ -608,7 +612,7 @@ inline SEXP sfc_multipoint(
   int start;
   int end;
   if( n_lines == 1 ) {
-    sfc( 0 ) = sfheaders::sfg::sfg_multipoint( df, geometry_cols );  // #nocov
+    sfc( 0 ) = sfheaders::sfg::sfg_multipoint( df, geometry_cols, xyzm );  // #nocov
   } else {
 
     R_xlen_t i;
@@ -618,7 +622,7 @@ inline SEXP sfc_multipoint(
       end = line_positions(i, 1);
 
       Rcpp::NumericMatrix m = sfheaders::shapes::get_listMat( df, geometry_cols, start, end );  // returns a matrix
-      sfc( i ) = sfheaders::sfg::sfg_multipoint( m );
+      sfc( i ) = sfheaders::sfg::sfg_multipoint( m, xyzm );
     }
   }
 
@@ -628,84 +632,92 @@ inline SEXP sfc_multipoint(
 inline SEXP sfc_multipoint(
     Rcpp::DataFrame& df,
     Rcpp::IntegerVector& geometry_cols,
-    SEXP& line_ids
+    SEXP& line_ids,
+    std::string xyzm
 ) {
   Rcpp::IntegerMatrix line_positions = sfheaders::utils::id_positions( line_ids );
-  return sfc_multipoint( df, geometry_cols, line_positions );
+  return sfc_multipoint( df, geometry_cols, line_positions, xyzm );
 }
 
 inline SEXP sfc_multipoint(
     Rcpp::IntegerMatrix& im,
     Rcpp::IntegerVector& geometry_cols,
-    int& multipoint_id
+    int& multipoint_id,
+    std::string xyzm
 ) {
   sfheaders::utils::column_exists( im, multipoint_id );
   Rcpp::IntegerVector line_ids = im( Rcpp::_, multipoint_id );
-  return sfc_multipoint( im, geometry_cols, line_ids );
+  return sfc_multipoint( im, geometry_cols, line_ids, xyzm );
 }
 
 inline SEXP sfc_multipoint(
     Rcpp::NumericMatrix& nm,
     Rcpp::IntegerVector& geometry_cols,
-    int& multipoint_id
+    int& multipoint_id,
+    std::string xyzm
 ) {
   sfheaders::utils::column_exists( nm, multipoint_id );
   Rcpp::NumericVector line_ids = nm( Rcpp::_, multipoint_id );
-  return sfc_multipoint( nm, geometry_cols, line_ids );
+  return sfc_multipoint( nm, geometry_cols, line_ids, xyzm );
 }
 
 // #nocov start
 inline SEXP sfc_multipoint(
     Rcpp::IntegerMatrix& im,
     Rcpp::StringVector& geometry_cols,
-    Rcpp::String& multipoint_id
+    Rcpp::String& multipoint_id,
+    std::string xyzm
 ) {
   Rcpp::DataFrame df = Rcpp::as< Rcpp::DataFrame >( im );
   SEXP line_ids = df[ multipoint_id ];
-  return sfc_multipoint( df, geometry_cols, line_ids );
+  return sfc_multipoint( df, geometry_cols, line_ids, xyzm );
 }
 
 inline SEXP sfc_multipoint(
     Rcpp::NumericMatrix& nm,
     Rcpp::StringVector& geometry_cols,
-    Rcpp::String& multipoint_id
+    Rcpp::String& multipoint_id,
+    std::string xyzm
 ) {
   Rcpp::DataFrame df = Rcpp::as< Rcpp::DataFrame >( nm );
   SEXP line_ids = df[ multipoint_id ];
-  return sfc_multipoint( df, geometry_cols, line_ids );
+  return sfc_multipoint( df, geometry_cols, line_ids, xyzm );
 }
 // #nocov end
 
 inline SEXP sfc_multipoint(
     Rcpp::DataFrame& df,
     Rcpp::StringVector& geometry_cols,
-    Rcpp::String& multipoint_id
+    Rcpp::String& multipoint_id,
+    std::string xyzm
 ) {
   SEXP line_ids = df[ multipoint_id ];
-  return sfc_multipoint( df, geometry_cols, line_ids );
+  return sfc_multipoint( df, geometry_cols, line_ids, xyzm );
 }
 
 inline SEXP sfc_multipoint(
     Rcpp::DataFrame& df,
     Rcpp::IntegerVector& geometry_cols,
-    int& multipoint_id
+    int& multipoint_id,
+    std::string xyzm
 ) {
   sfheaders::utils::column_exists( df, multipoint_id );
   SEXP line_ids = df[ multipoint_id ];
-  return sfc_multipoint( df, geometry_cols, line_ids );
+  return sfc_multipoint( df, geometry_cols, line_ids, xyzm );
 }
 
 inline SEXP sfc_multipoint(
     SEXP& x,
     Rcpp::IntegerVector& geometry_cols,
-    int& multipoint_id
+    int& multipoint_id,
+    std::string xyzm
 ) {
   switch( TYPEOF( x ) ) {
   case INTSXP: {
     if( Rf_isMatrix( x ) ) {
     SEXP xc = Rcpp::clone( x );
     Rcpp::IntegerMatrix im = Rcpp::as< Rcpp::IntegerMatrix >( xc );
-    return sfc_multipoint( im, geometry_cols, multipoint_id );
+    return sfc_multipoint( im, geometry_cols, multipoint_id, xyzm );
     // } else {
     // Rcpp::IntegerVector iv = Rcpp::as< Rcpp::IntegerVector >( x );
     // return sfc_multipoint( iv, geometry_cols, multipoint_id );
@@ -715,7 +727,7 @@ inline SEXP sfc_multipoint(
     if( Rf_isMatrix( x ) ) {
     SEXP xc = Rcpp::clone( x );
     Rcpp::NumericMatrix nm = Rcpp::as< Rcpp::NumericMatrix >( xc );
-    return sfc_multipoint( nm, geometry_cols, multipoint_id );
+    return sfc_multipoint( nm, geometry_cols, multipoint_id, xyzm );
     // } else {
     //   Rcpp::NumericVector nv = Rcpp::as< Rcpp::NumericVector >( x );
     //   return sfc_multipoint( nv, geometry_cols, multipoint_id );
@@ -724,7 +736,7 @@ inline SEXP sfc_multipoint(
   case VECSXP: {
     if( Rf_inherits( x, "data.frame" ) ) {
     Rcpp::DataFrame df = Rcpp::as< Rcpp::DataFrame >( x );
-    return sfc_multipoint( df, geometry_cols, multipoint_id );
+    return sfc_multipoint( df, geometry_cols, multipoint_id, xyzm );
   }
   }
   default: {
@@ -739,7 +751,8 @@ inline SEXP sfc_multipoint(
 inline SEXP sfc_multipoint(
     SEXP& x,
     Rcpp::StringVector& geometry_cols,
-    Rcpp::String& multipoint_id
+    Rcpp::String& multipoint_id,
+    std::string xyzm
 ) {
   switch( TYPEOF( x ) ) {
   case INTSXP: {
@@ -747,7 +760,7 @@ inline SEXP sfc_multipoint(
     if( Rf_isMatrix( x ) ) {
     SEXP xc = Rcpp::clone( x );
     Rcpp::IntegerMatrix im = Rcpp::as< Rcpp::IntegerMatrix >( xc );
-    return sfc_multipoint( im, geometry_cols, multipoint_id );
+    return sfc_multipoint( im, geometry_cols, multipoint_id, xyzm );
     // } else {
     //   Rcpp::IntegerVector iv = Rcpp::as< Rcpp::IntegerVector >( x );
     //   return sfc_multipoint( iv, geometry_cols, multipoint_id );
@@ -757,7 +770,7 @@ inline SEXP sfc_multipoint(
     if( Rf_isMatrix( x ) ) {
     SEXP xc = Rcpp::clone( x );
     Rcpp::NumericMatrix nm = Rcpp::as< Rcpp::NumericMatrix >( xc );
-    return sfc_multipoint( nm, geometry_cols, multipoint_id );
+    return sfc_multipoint( nm, geometry_cols, multipoint_id, xyzm );
     // } else {
     //   Rcpp::NumericVector nv = Rcpp::as< Rcpp::NumericVector >( x );
     //   return sfc_multipoint( nv, geometry_cols, multipoint_id );
@@ -767,7 +780,7 @@ inline SEXP sfc_multipoint(
   case VECSXP: {
     if( Rf_inherits( x, "data.frame" ) ) {
     Rcpp::DataFrame df = Rcpp::as< Rcpp::DataFrame >( x );
-    return sfc_multipoint( df, geometry_cols, multipoint_id );
+    return sfc_multipoint( df, geometry_cols, multipoint_id, xyzm );
   }
   }
   default: {
@@ -783,24 +796,25 @@ inline SEXP sfc_multipoint(
 inline SEXP sfc_multipoint(
     SEXP& x,
     SEXP& geometry_cols,
-    SEXP& multipoint_id
+    SEXP& multipoint_id,
+    std::string xyzm
 ) {
 
   if( Rf_isNull( geometry_cols ) && Rf_isNull( multipoint_id ) ) {
 
-    return sfc_multipoint( x );
+    return sfc_multipoint( x, xyzm );
 
   } else if ( !Rf_isNull( geometry_cols ) && Rf_isNull( multipoint_id ) ) {
 
     // make the geometry cols all the other columns??
-    return sfc_multipoint( x, geometry_cols );
+    return sfc_multipoint( x, geometry_cols, xyzm );
 
   } else if ( Rf_isNull( geometry_cols ) && !Rf_isNull( multipoint_id ) ) {
 
     SEXP other_cols = sfheaders::utils::other_columns( x, multipoint_id );
     sfheaders::utils::geometry_column_check( other_cols );
 
-    return sfc_multipoint( x, other_cols, multipoint_id );
+    return sfc_multipoint( x, other_cols, multipoint_id, xyzm );
 
   } else {
     // we have all the columns sorted, so we can now get their types, calcaulte ranges/bbox etc
@@ -816,14 +830,14 @@ inline SEXP sfc_multipoint(
       Rcpp::IntegerVector iv_geometry_cols = Rcpp::as< Rcpp::IntegerVector >( geometry_cols );
       Rcpp::IntegerVector iv_multipoint_id_col = Rcpp::as< Rcpp::IntegerVector >( multipoint_id );
       int i_multipoint_id_col = iv_multipoint_id_col[0];
-      return sfc_multipoint( x, iv_geometry_cols, i_multipoint_id_col );
+      return sfc_multipoint( x, iv_geometry_cols, i_multipoint_id_col, xyzm );
 
     }
     case STRSXP: {
       Rcpp::StringVector sv_geometry_cols = Rcpp::as< Rcpp::StringVector >( geometry_cols );
       Rcpp::StringVector sv_multipoint_id_col = Rcpp::as< Rcpp::StringVector >( multipoint_id );
       Rcpp::String s_multipoint_id_col = sv_multipoint_id_col[0];
-      return sfc_multipoint( x, sv_geometry_cols, s_multipoint_id_col );
+      return sfc_multipoint( x, sv_geometry_cols, s_multipoint_id_col, xyzm );
     }
     default: {
       Rcpp::stop("sfheaders - unsupported multipoint type");  // #nocov
