@@ -7,7 +7,7 @@
 namespace sfheaders {
 namespace sfg {
 
-  inline Rcpp::List sfg_linestrings( Rcpp::List& lst ) {
+  inline Rcpp::List sfg_linestrings( Rcpp::List& lst, std::string xyzm ) {
     R_xlen_t n = lst.size();
     // Rcpp::Rcout << "linestring sizes: " << n << std::endl;
     R_xlen_t i;
@@ -15,7 +15,7 @@ namespace sfg {
 
     for( i = 0; i < n; ++i ) {
       SEXP x = lst[i];
-      sfcs[i] = sfheaders::sfg::sfg_linestring( x );
+      sfcs[i] = sfheaders::sfg::sfg_linestring( x, xyzm );
     }
     return sfcs;
   }
