@@ -13,6 +13,7 @@ namespace sfg {
 // multipolygon is a list of list of linestrings (matrices)
 inline SEXP sfg_multipolygon(
     Rcpp::IntegerMatrix& im,
+    std::string xyzm,
     bool close = true
 ) {
   Rcpp::List p( 1 );
@@ -20,13 +21,14 @@ inline SEXP sfg_multipolygon(
   p[0] = sfheaders::polygon_utils::close_polygon( im, close );;
   mp[0] = p;
   R_xlen_t n_col = im.ncol();
-  sfheaders::sfg::make_sfg( mp, n_col, sfheaders::sfg::SFG_MULTIPOLYGON );
+  sfheaders::sfg::make_sfg( mp, n_col, sfheaders::sfg::SFG_MULTIPOLYGON, xyzm );
 
   return mp;
 }
 
 inline SEXP sfg_multipolygon(
     Rcpp::NumericMatrix& nm,
+    std::string xyzm,
     bool close = true
 ) {
   Rcpp::List p( 1 );
@@ -34,13 +36,14 @@ inline SEXP sfg_multipolygon(
   p[0] = sfheaders::polygon_utils::close_polygon( nm, close );;
   mp[0] = p;
   R_xlen_t n_col = nm.ncol();
-  sfheaders::sfg::make_sfg( mp, n_col, sfheaders::sfg::SFG_MULTIPOLYGON );
+  sfheaders::sfg::make_sfg( mp, n_col, sfheaders::sfg::SFG_MULTIPOLYGON, xyzm );
 
   return mp;
 }
 
 inline SEXP sfg_multipolygon(
     Rcpp::DataFrame& df,
+    std::string xyzm,
     bool close = true
 ) {
 
@@ -50,7 +53,7 @@ inline SEXP sfg_multipolygon(
   p[0] = sfheaders::polygon_utils::close_polygon( nm, close );;
   mp[0] = p;
   R_xlen_t n_col = nm.ncol();
-  sfheaders::sfg::make_sfg( mp, n_col, sfheaders::sfg::SFG_MULTIPOLYGON );
+  sfheaders::sfg::make_sfg( mp, n_col, sfheaders::sfg::SFG_MULTIPOLYGON, xyzm );
 
   return mp;
 }
@@ -59,11 +62,12 @@ inline SEXP sfg_multipolygon(
 
 inline SEXP sfg_multipolygon(
     Rcpp::List& lst,
+    std::string xyzm,
     bool close = true
 ) {
 
   lst = sfheaders::polygon_utils::close_polygon( lst, close );
-  sfheaders::sfg::make_sfg( lst, sfheaders::sfg::SFG_MULTIPOLYGON );
+  sfheaders::sfg::make_sfg( lst, sfheaders::sfg::SFG_MULTIPOLYGON, xyzm );
   return lst;
 }
 
@@ -71,55 +75,61 @@ inline SEXP sfg_multipolygon(
 inline SEXP sfg_multipolygon(
     Rcpp::DataFrame& df,
     Rcpp::IntegerVector& cols,
+    std::string xyzm,
     bool close = true
 ) {
   Rcpp::NumericMatrix nm = sfheaders::shapes::get_mat( df, cols );
-  return sfg_multipolygon( nm, close );
+  return sfg_multipolygon( nm, xyzm, close);
 }
 
 inline SEXP sfg_multipolygon(
     Rcpp::DataFrame& df,
     Rcpp::StringVector& cols,
+    std::string xyzm,
     bool close = true
 ) {
   Rcpp::NumericMatrix nm = sfheaders::shapes::get_mat( df, cols );
-  return sfg_multipolygon( nm, close );
+  return sfg_multipolygon( nm, xyzm, close);
 }
 
 inline SEXP sfg_multipolygon(
     Rcpp::NumericMatrix& nm,
     Rcpp::IntegerVector& cols,
+    std::string xyzm,
     bool close = true
 ) {
   Rcpp::NumericMatrix nm2 = sfheaders::shapes::get_mat( nm, cols );
-  return sfg_multipolygon( nm2, close );
+  return sfg_multipolygon( nm2, xyzm, close);
 }
 
 inline SEXP sfg_multipolygon(
     Rcpp::NumericMatrix& nm,
     Rcpp::StringVector& cols,
+    std::string xyzm,
     bool close = true
 ) {
   Rcpp::NumericMatrix nm2 = sfheaders::shapes::get_mat( nm, cols );
-  return sfg_multipolygon( nm2, close );
+  return sfg_multipolygon( nm2, xyzm, close);
 }
 
 inline SEXP sfg_multipolygon(
     Rcpp::IntegerMatrix& im,
     Rcpp::IntegerVector& cols,
+    std::string xyzm,
     bool close = true
 ) {
   Rcpp::IntegerMatrix im2 = sfheaders::shapes::get_mat( im, cols );
-  return sfg_multipolygon( im2, close );
+  return sfg_multipolygon( im2, xyzm, close);
 }
 
 inline SEXP sfg_multipolygon(
     Rcpp::IntegerMatrix& im,
     Rcpp::StringVector& cols,
+    std::string xyzm,
     bool close = true
 ) {
   Rcpp::IntegerMatrix im2 = sfheaders::shapes::get_mat( im, cols );
-  return sfg_multipolygon( im2, close );
+  return sfg_multipolygon( im2, xyzm, close);
 }
 
 // We're still on single sfg objects.
@@ -128,60 +138,66 @@ inline SEXP sfg_multipolygon(
     Rcpp::DataFrame& df,
     Rcpp::IntegerVector& cols,
     int& id_col,
+    std::string xyzm,
     bool close = true
 ) {
   Rcpp::List lst = sfheaders::shapes::get_listMat( df, cols, id_col );
-  return sfg_multipolygon( lst, close );
+  return sfg_multipolygon( lst, xyzm, close);
 }
 
 inline SEXP sfg_multipolygon(
     Rcpp::DataFrame& df,
     Rcpp::StringVector& cols,
     Rcpp::String& id_col,
+    std::string xyzm,
     bool close = true
 ) {
   Rcpp::List lst = sfheaders::shapes::get_listMat( df, cols, id_col );
-  return sfg_multipolygon( lst, close );
+  return sfg_multipolygon( lst, xyzm, close);
 }
 
 inline SEXP sfg_multipolygon(
     Rcpp::NumericMatrix& nm,
     Rcpp::IntegerVector& cols,
     int& id_col,
+    std::string xyzm,
     bool close = true
 ) {
   Rcpp::List lst = sfheaders::shapes::get_listMat( nm, cols, id_col );
-  return sfg_multipolygon( lst, close );
+  return sfg_multipolygon( lst, xyzm, close);
 }
 
 inline SEXP sfg_multipolygon(
     Rcpp::NumericMatrix& nm,
     Rcpp::StringVector& cols,
     Rcpp::String& id_col,
+    std::string xyzm,
     bool close = true
 ) {
   Rcpp::List lst = sfheaders::shapes::get_listMat( nm, cols, id_col );
-  return sfg_multipolygon( lst, close );
+  return sfg_multipolygon( lst, xyzm, close);
 }
 
 inline SEXP sfg_multipolygon(
     Rcpp::IntegerMatrix& im,
     Rcpp::IntegerVector& cols,
     int& id_col,
+    std::string xyzm,
     bool close = true
 ) {
   Rcpp::List lst = sfheaders::shapes::get_listMat( im, cols, id_col );
-  return sfg_multipolygon( lst, close );
+  return sfg_multipolygon( lst, xyzm, close);
 }
 
 inline SEXP sfg_multipolygon(
     Rcpp::IntegerMatrix& im,
     Rcpp::StringVector& cols,
     Rcpp::String& id_col,
+    std::string xyzm,
     bool close = true
 ) {
   Rcpp::List lst = sfheaders::shapes::get_listMat( im, cols, id_col );
-  return sfg_multipolygon( lst, close );
+  return sfg_multipolygon( lst, xyzm, close);
 }
 
   inline SEXP sfg_multipolygon(
@@ -189,10 +205,11 @@ inline SEXP sfg_multipolygon(
     Rcpp::IntegerVector& geometry_cols,
     int& polygon_id_column,
     int& line_id_column,
+    std::string xyzm,
     bool close = true
   ) {
     Rcpp::List mp = sfheaders::shapes::get_listListMat( im, geometry_cols, polygon_id_column, line_id_column );
-    return sfg_multipolygon( mp, close );
+    return sfg_multipolygon( mp, xyzm, close);
   }
 
   inline SEXP sfg_multipolygon(
@@ -200,10 +217,11 @@ inline SEXP sfg_multipolygon(
       Rcpp::StringVector& geometry_cols,
       Rcpp::String& polygon_id_column,
       Rcpp::String& line_id_column,
+      std::string xyzm,
       bool close = true
   ) {
     Rcpp::List mp = sfheaders::shapes::get_listListMat( im, geometry_cols, polygon_id_column, line_id_column );
-    return sfg_multipolygon( mp, close );
+    return sfg_multipolygon( mp, xyzm, close);
   }
 
   inline SEXP sfg_multipolygon(
@@ -211,10 +229,11 @@ inline SEXP sfg_multipolygon(
       Rcpp::IntegerVector& geometry_cols,
       int& polygon_id_column,
       int& line_id_column,
+      std::string xyzm,
       bool close = true
   ) {
     Rcpp::List mp = sfheaders::shapes::get_listListMat( nm, geometry_cols, polygon_id_column, line_id_column );
-    return sfg_multipolygon( mp, close );
+    return sfg_multipolygon( mp, xyzm, close);
   }
 
   inline SEXP sfg_multipolygon(
@@ -222,10 +241,11 @@ inline SEXP sfg_multipolygon(
       Rcpp::StringVector& geometry_cols,
       Rcpp::String& polygon_id_column,
       Rcpp::String& line_id_column,
+      std::string xyzm,
       bool close = true
   ) {
     Rcpp::List mp = sfheaders::shapes::get_listListMat( nm, geometry_cols, polygon_id_column, line_id_column );
-    return sfg_multipolygon( mp, close );
+    return sfg_multipolygon( mp, xyzm, close);
   }
 
   inline SEXP sfg_multipolygon(
@@ -233,10 +253,11 @@ inline SEXP sfg_multipolygon(
       Rcpp::IntegerVector& geometry_cols,
       int& polygon_id_column,
       int& line_id_column,
+      std::string xyzm,
       bool close = true
   ) {
     Rcpp::List mp = sfheaders::shapes::get_listListMat( df, geometry_cols, polygon_id_column, line_id_column );
-    return sfg_multipolygon( mp, close );
+    return sfg_multipolygon( mp, xyzm, close);
   }
 
 
@@ -245,16 +266,18 @@ inline SEXP sfg_multipolygon(
       Rcpp::StringVector& geometry_cols,
       Rcpp::String& polygon_id_column,
       Rcpp::String& line_id_column,
+      std::string xyzm,
       bool close = true
   ) {
     Rcpp::List mp = sfheaders::shapes::get_listListMat( df, geometry_cols, polygon_id_column, line_id_column );
-    return sfg_multipolygon( mp, close );
+    return sfg_multipolygon( mp, xyzm, close);
   }
 
 
 inline SEXP sfg_multipolygon(
     SEXP& x,
     Rcpp::IntegerVector& cols,
+    std::string xyzm,
     bool close = true
 ) {
   switch( TYPEOF( x ) ) {
@@ -263,19 +286,19 @@ inline SEXP sfg_multipolygon(
     Rcpp::stop("sfheaders - expecting a matrix"); // #nocov
   }
     Rcpp::IntegerMatrix im = Rcpp::as< Rcpp::IntegerMatrix >( x );
-    return sfg_multipolygon( im, cols, close );
+    return sfg_multipolygon( im, cols, xyzm, close);
   }
   case REALSXP: {
     if( !Rf_isMatrix( x ) ) {
     Rcpp::stop("sfheaders - expecting a matrix"); // #nocov
   }
     Rcpp::NumericMatrix nm = Rcpp::as< Rcpp::NumericMatrix >( x );
-    return sfg_multipolygon( nm, cols, close );
+    return sfg_multipolygon( nm, cols, xyzm, close);
   }
   case VECSXP: {
     if( Rf_inherits( x, "data.frame") ) {
     Rcpp::DataFrame df = Rcpp::as< Rcpp::DataFrame >( x );
-    return sfg_multipolygon( df, cols, close );
+    return sfg_multipolygon( df, cols, xyzm, close);
   } // else default
   }
   default: {
@@ -292,6 +315,7 @@ inline SEXP sfg_multipolygon(
     SEXP& x,
     Rcpp::IntegerVector& cols,
     int& line_id,
+    std::string xyzm,
     bool close = true
 ) {
   switch( TYPEOF( x ) ) {
@@ -300,19 +324,19 @@ inline SEXP sfg_multipolygon(
     Rcpp::stop("sfheaders - expecting a matrix"); // #nocov
   }
     Rcpp::IntegerMatrix im = Rcpp::as< Rcpp::IntegerMatrix >( x );
-    return sfg_multipolygon( im, cols, line_id, close );
+    return sfg_multipolygon( im, cols, line_id, xyzm, close);
   }
   case REALSXP: {
     if( !Rf_isMatrix( x ) ) {
     Rcpp::stop("sfheaders - expecting a matrix"); // #nocov
   }
     Rcpp::NumericMatrix nm = Rcpp::as< Rcpp::NumericMatrix >( x );
-    return sfg_multipolygon( nm, cols, line_id, close );
+    return sfg_multipolygon( nm, cols, line_id, xyzm, close);
   }
   case VECSXP: {
     if( Rf_inherits( x, "data.frame") ) {
     Rcpp::DataFrame df = Rcpp::as< Rcpp::DataFrame >( x );
-    return sfg_multipolygon( df, cols, line_id, close );
+    return sfg_multipolygon( df, cols, line_id, xyzm, close);
   } // else default
   }
   default: {
@@ -362,6 +386,7 @@ inline SEXP sfg_multipolygon(
 inline SEXP sfg_multipolygon(
     SEXP& x,
     Rcpp::StringVector& cols,
+    std::string xyzm,
     bool close = true
 ) {
   switch( TYPEOF( x ) ) {
@@ -370,19 +395,19 @@ inline SEXP sfg_multipolygon(
     Rcpp::stop("sfheaders - expecting a matrix"); // #nocov
   }
     Rcpp::IntegerMatrix im = Rcpp::as< Rcpp::IntegerMatrix >( x );
-    return sfg_multipolygon( im, cols, close );
+    return sfg_multipolygon( im, cols, xyzm, close);
   }
   case REALSXP: {
     if( !Rf_isMatrix( x ) ) {
     Rcpp::stop("sfheaders - expecting a matrix"); // #nocov
   }
     Rcpp::NumericMatrix nm = Rcpp::as< Rcpp::NumericMatrix >( x );
-    return sfg_multipolygon( nm, cols, close );
+    return sfg_multipolygon( nm, cols, xyzm, close);
   }
   case VECSXP: {
     if( Rf_inherits( x, "data.frame") ) {
     Rcpp::DataFrame df = Rcpp::as< Rcpp::DataFrame >( x );
-    return sfg_multipolygon( df, cols, close );
+    return sfg_multipolygon( df, cols, xyzm, close);
   } // else default
   }
   default: {
@@ -431,6 +456,7 @@ inline SEXP sfg_multipolygon(
     SEXP& x,
     Rcpp::StringVector& cols,
     Rcpp::String& line_id,
+    std::string xyzm,
     bool close = true
 ) {
   switch( TYPEOF( x ) ) {
@@ -439,19 +465,19 @@ inline SEXP sfg_multipolygon(
     Rcpp::stop("sfheaders - expecting a matrix"); // #nocov
   }
     Rcpp::IntegerMatrix im = Rcpp::as< Rcpp::IntegerMatrix >( x );
-    return sfg_multipolygon( im, cols, line_id, close );
+    return sfg_multipolygon( im, cols, line_id, xyzm, close);
   }
   case REALSXP: {
     if( !Rf_isMatrix( x ) ) {
     Rcpp::stop("sfheaders - expecting a matrix"); // #nocov
   }
     Rcpp::NumericMatrix nm = Rcpp::as< Rcpp::NumericMatrix >( x );
-    return sfg_multipolygon( nm, cols, line_id, close );
+    return sfg_multipolygon( nm, cols, line_id, xyzm, close);
   }
   case VECSXP: {
     if( Rf_inherits( x, "data.frame") ) {
     Rcpp::DataFrame df = Rcpp::as< Rcpp::DataFrame >( x );
-    return sfg_multipolygon( df, cols, line_id, close );
+    return sfg_multipolygon( df, cols, line_id, xyzm, close);
   } // else default
   }
   default: {
@@ -465,6 +491,7 @@ inline SEXP sfg_multipolygon(
 
 inline SEXP sfg_multipolygon(
     SEXP& x,
+    std::string xyzm,
     bool close = true
 ) {
 
@@ -474,22 +501,22 @@ inline SEXP sfg_multipolygon(
     Rcpp::stop("sfheaders - expecting a matrix"); // #nocov
   }
     Rcpp::IntegerMatrix im = Rcpp::as< Rcpp::IntegerMatrix >( x );
-    return sfg_multipolygon( im, close );
+    return sfg_multipolygon( im, xyzm, close);
   }
   case REALSXP: {
     if( !Rf_isMatrix( x ) ) {
     Rcpp::stop("sfheaders - expecting a matrix"); // #nocov
   }
     Rcpp::NumericMatrix nm = Rcpp::as< Rcpp::NumericMatrix >( x );
-    return sfg_multipolygon( nm, close );
+    return sfg_multipolygon( nm, xyzm, close);
   }
   case VECSXP: {
     if( Rf_inherits( x, "data.frame") ) {
     Rcpp::DataFrame df = Rcpp::as< Rcpp::DataFrame >( x );
-    return sfg_multipolygon( df, close );
+    return sfg_multipolygon( df, xyzm, close);
   } else {
     Rcpp::List lst = Rcpp::as< Rcpp::List >( x );
-    return sfg_multipolygon( lst, close );
+    return sfg_multipolygon( lst, xyzm, close);
   }
   }
   default: {
@@ -503,20 +530,21 @@ inline SEXP sfg_multipolygon(
 inline SEXP sfg_multipolygon(
     SEXP& x,
     SEXP& cols,
+    std::string xyzm,
     bool close = true
 ) {
   if( Rf_isNull( cols ) ) {
-    return sfg_multipolygon( x, close );
+    return sfg_multipolygon( x, xyzm, close);
   }
   switch( TYPEOF( cols ) ) {
   case REALSXP: {}
   case INTSXP: {
     Rcpp::IntegerVector iv = Rcpp::as< Rcpp::IntegerVector >( cols );
-    return sfg_multipolygon( x, iv, close );
+    return sfg_multipolygon( x, iv, xyzm, close);
   }
   case STRSXP: {
     Rcpp::StringVector sv = Rcpp::as< Rcpp::StringVector >( cols );
-    return sfg_multipolygon( x, sv, close );
+    return sfg_multipolygon( x, sv, xyzm, close);
   }
   default: {
     Rcpp::stop("sfheaders - unknown column types");  // #nocov
@@ -530,13 +558,14 @@ inline SEXP sfg_multipolygon(
     SEXP& x,
     SEXP& cols,
     Rcpp::String& line_id,
+    std::string xyzm,
     bool close = true
 ) {
   if( Rf_isNull( cols ) ) {
     Rcpp::StringVector id_cols( 1 );
     id_cols[0] = line_id;
     SEXP other_cols = sfheaders::utils::other_columns( x, id_cols );
-    return sfg_multipolygon( x, other_cols, line_id, close );
+    return sfg_multipolygon( x, other_cols, line_id, xyzm, close);
   }
   switch( TYPEOF( cols ) ) {
   // case REALSXP: {}
@@ -546,7 +575,7 @@ inline SEXP sfg_multipolygon(
   // }
   case STRSXP: {
     Rcpp::StringVector sv = Rcpp::as< Rcpp::StringVector >( cols );
-    return sfg_multipolygon( x, sv, line_id, close );
+    return sfg_multipolygon( x, sv, line_id, xyzm, close);
   }
   default: {
     Rcpp::stop("sfheaders - unknown column types");
@@ -560,6 +589,7 @@ inline SEXP sfg_multipolygon(
     SEXP& x,
     SEXP& cols,
     int& line_id,
+    std::string xyzm,
     bool close = true
 ) {
 
@@ -567,13 +597,13 @@ inline SEXP sfg_multipolygon(
     Rcpp::IntegerVector id_cols( 1 );
     id_cols[0] = line_id;
     SEXP other_cols = sfheaders::utils::other_columns( x, id_cols );
-    return sfg_multipolygon( x, other_cols, line_id, close );
+    return sfg_multipolygon( x, other_cols, line_id, xyzm, close);
   }
   switch( TYPEOF( cols ) ) {
   case REALSXP: {}
   case INTSXP: {
     Rcpp::IntegerVector iv = Rcpp::as< Rcpp::IntegerVector >( cols );
-    return sfg_multipolygon( x, iv, line_id, close );
+    return sfg_multipolygon( x, iv, line_id, xyzm, close);
   }
   // case STRSXP: {
   //   Rcpp::StringVector sv = Rcpp::as< Rcpp::StringVector >( cols );
@@ -591,10 +621,11 @@ inline SEXP sfg_multipolygon(
   Rcpp::IntegerVector geometry_cols,
   int& polygon_id,
   int& line_id,
+  std::string xyzm,
   bool close = true
 ) {
   Rcpp::List mp = sfheaders::shapes::get_listListMat( x, geometry_cols, polygon_id, line_id );
-  return sfg_multipolygon( mp, close );
+  return sfg_multipolygon( mp, xyzm, close);
 }
 
 inline SEXP sfg_multipolygon(
@@ -602,10 +633,11 @@ inline SEXP sfg_multipolygon(
     Rcpp::StringVector geometry_cols,
     Rcpp::String& polygon_id,
     Rcpp::String& line_id,
+    std::string xyzm,
     bool close = true
 ) {
   Rcpp::List mp = sfheaders::shapes::get_listListMat( x, geometry_cols, polygon_id, line_id );
-  return sfg_multipolygon( mp, close );
+  return sfg_multipolygon( mp, xyzm, close);
 }
 
 
@@ -614,6 +646,7 @@ inline SEXP sfg_multipolygon(
     SEXP& cols,
     int& polygon_id,
     int& line_id,
+    std::string xyzm,
     bool close = true
 ) {
 
@@ -623,7 +656,7 @@ inline SEXP sfg_multipolygon(
     id_cols[0] = polygon_id;
     id_cols[1] = line_id;
     SEXP other_cols = sfheaders::utils::other_columns( x, id_cols );
-    return sfg_multipolygon( x, other_cols, polygon_id, line_id, close );
+    return sfg_multipolygon( x, other_cols, polygon_id, line_id, xyzm, close);
   }
   // #nocov end
 
@@ -631,7 +664,7 @@ inline SEXP sfg_multipolygon(
   case REALSXP: {}
   case INTSXP: {
     Rcpp::IntegerVector geometry_cols = Rcpp::as< Rcpp::IntegerVector >( cols );
-    return sfg_multipolygon( x, geometry_cols, polygon_id, line_id, close );
+    return sfg_multipolygon( x, geometry_cols, polygon_id, line_id, xyzm, close);
   }
   // case STRSXP: {
   //   Rcpp::StringVector geometry_cols = Rcpp::as< Rcpp::StringVector >( cols );
@@ -648,6 +681,7 @@ inline SEXP sfg_multipolygon(
     SEXP& cols,
     Rcpp::String& polygon_id,
     Rcpp::String& line_id,
+    std::string xyzm,
     bool close = true
 ) {
 
@@ -657,7 +691,7 @@ inline SEXP sfg_multipolygon(
     id_cols[0] = polygon_id;
     id_cols[1] = line_id;
     SEXP other_cols = sfheaders::utils::other_columns( x, id_cols );
-    return sfg_multipolygon( x, other_cols, polygon_id, line_id, close );
+    return sfg_multipolygon( x, other_cols, polygon_id, line_id, xyzm, close);
   }
   // #nocov end
 
@@ -669,7 +703,7 @@ inline SEXP sfg_multipolygon(
   // }
   case STRSXP: {
     Rcpp::StringVector geometry_cols = Rcpp::as< Rcpp::StringVector >( cols );
-    return sfg_multipolygon( x, geometry_cols, polygon_id, line_id, close );
+    return sfg_multipolygon( x, geometry_cols, polygon_id, line_id, xyzm, close);
   }
   default: {
     Rcpp::stop("sfheaders - unknown column types");
@@ -683,10 +717,11 @@ inline SEXP sfg_multipolygon(
     SEXP& cols,
     SEXP& polygon_id,
     SEXP& line_id,
+    std::string xyzm,
     bool close = true
 ) {
   if( Rf_isNull( polygon_id ) && Rf_isNull( line_id ) ) {
-    return sfg_multipolygon( x, cols, close );
+    return sfg_multipolygon( x, cols, xyzm, close);
   }
 
   // at this point one or both of polygon_id and line_id will not be null
@@ -694,7 +729,7 @@ inline SEXP sfg_multipolygon(
   if( Rf_isNull( cols ) ) {
     // make them the other columns
     SEXP other_cols = sfheaders::utils::other_columns(x, polygon_id, line_id );
-    return sfg_multipolygon( x, other_cols, polygon_id, line_id, close );
+    return sfg_multipolygon( x, other_cols, polygon_id, line_id, xyzm, close);
   }
 
   if( Rf_isNull( polygon_id ) && !Rf_isNull( line_id ) ) {
@@ -706,7 +741,7 @@ inline SEXP sfg_multipolygon(
 
     Rcpp::List sfg(1);
     sfg[0] = sfheaders::shapes::get_listMat( x, cols, line_id );
-    return sfg_multipolygon( sfg, close );
+    return sfg_multipolygon( sfg, xyzm, close);
   }
 
   if( !Rf_isNull( polygon_id ) && Rf_isNull( line_id ) ) {
@@ -714,7 +749,7 @@ inline SEXP sfg_multipolygon(
     // with one or more polygons,
     // but each polygon has a single line
     SEXP line_id2 = polygon_id;
-    return sfg_multipolygon( x, cols, polygon_id, line_id2, close );
+    return sfg_multipolygon( x, cols, polygon_id, line_id2, xyzm, close);
   }
 
   // otherwise they are both provided
@@ -729,7 +764,7 @@ inline SEXP sfg_multipolygon(
     Rcpp::IntegerVector iv_polygon = Rcpp::as< Rcpp::IntegerVector >( polygon_id );
     int il = iv_line[0];
     int ip = iv_polygon[0];
-    return sfg_multipolygon( x, cols, ip, il, close );
+    return sfg_multipolygon( x, cols, ip, il, xyzm, close);
   }
   case STRSXP: {
     Rcpp::StringVector sv_line = Rcpp::as< Rcpp::StringVector >( line_id );
@@ -738,7 +773,7 @@ inline SEXP sfg_multipolygon(
     Rcpp::String sl = sv_line[0];
     Rcpp::String sp = sv_polygon[0];
     //return Rcpp::List();
-    return sfg_multipolygon( x, cols, sp, sl, close );
+    return sfg_multipolygon( x, cols, sp, sl, xyzm, close);
   }
   default: {
     Rcpp::stop("sfheaders - unknown column types");  // #nocov
@@ -750,6 +785,7 @@ inline SEXP sfg_multipolygon(
   // only keep the outer-linestring / ring / matrix
   inline SEXP remove_multipolygon_holes(
       Rcpp::List& sfg_mp,
+      std::string xyzm,
       bool close = true
   ) {
     // loop over and only keep the first line
@@ -762,7 +798,7 @@ inline SEXP sfg_multipolygon(
       new_poly[ 0 ] = poly[ 0 ];
       res[ i ] = new_poly;
     }
-    return sfg_multipolygon( res, close );
+    return sfg_multipolygon( res, xyzm, close);
 
   }
 
