@@ -9,103 +9,113 @@
 namespace sfheaders {
 namespace sfg {
 
-  inline SEXP sfg_point( Rcpp::IntegerVector& iv ) {
-    sfheaders::sfg::make_sfg( iv, sfheaders::sfg::SFG_POINT );
+  inline SEXP sfg_point( Rcpp::IntegerVector& iv, std::string xyzm ) {
+    sfheaders::sfg::make_sfg( iv, sfheaders::sfg::SFG_POINT, xyzm );
     return iv;
   }
 
-  inline SEXP sfg_point( Rcpp::NumericVector& nv ) {
-    sfheaders::sfg::make_sfg( nv, sfheaders::sfg::SFG_POINT );
+  inline SEXP sfg_point( Rcpp::NumericVector& nv, std::string xyzm ) {
+    sfheaders::sfg::make_sfg( nv, sfheaders::sfg::SFG_POINT, xyzm );
     return nv;
   }
 
-  inline SEXP sfg_point( Rcpp::IntegerVector& iv, Rcpp::IntegerVector& cols ) {
+  inline SEXP sfg_point( Rcpp::IntegerVector& iv, Rcpp::IntegerVector& cols, std::string xyzm ) {
     Rcpp::IntegerVector iv2 = iv[ cols ];
-    return sfg_point( iv2 );
+    return sfg_point( iv2, xyzm );
   }
 
-  inline SEXP sfg_point( Rcpp::NumericVector& nv, Rcpp::IntegerVector& cols ) {
+  inline SEXP sfg_point( Rcpp::NumericVector& nv, Rcpp::IntegerVector& cols, std::string xyzm ) {
     Rcpp::NumericVector nv2 = nv[ cols ];
-    return sfg_point( nv2 );
+    return sfg_point( nv2, xyzm );
   }
 
   /*
    * assumes columns are in lon/lat/z/m order
    */
   inline SEXP sfg_point(
-      Rcpp::IntegerMatrix& im
+      Rcpp::IntegerMatrix& im,
+      std::string xyzm
   ) {
     Rcpp::IntegerVector iv = sfheaders::shapes::get_vec( im );
-    return sfg_point( iv );
+    return sfg_point( iv, xyzm );
   }
 
   inline SEXP sfg_point(
-      Rcpp::NumericMatrix& nm
+      Rcpp::NumericMatrix& nm,
+      std::string xyzm
   ) {
     Rcpp::NumericVector nv = sfheaders::shapes::get_vec( nm );
-    return sfg_point( nv );
+    return sfg_point( nv, xyzm );
   }
 
   inline SEXP sfg_point(
       Rcpp::IntegerMatrix& im,
-      Rcpp::IntegerVector& cols
+      Rcpp::IntegerVector& cols,
+      std::string xyzm
   ) {
     Rcpp::IntegerVector iv = sfheaders::shapes::get_vec( im, cols );
-    return sfg_point( iv );
+    return sfg_point( iv, xyzm );
   }
 
   inline SEXP sfg_point(
       Rcpp::IntegerMatrix& im,
-      Rcpp::StringVector& cols
+      Rcpp::StringVector& cols,
+      std::string xyzm
   ) {
     Rcpp::IntegerVector iv = sfheaders::shapes::get_vec( im, cols );
-    return sfg_point( iv );
+    return sfg_point( iv, xyzm );
   }
 
   inline SEXP sfg_point(
       Rcpp::NumericMatrix& nm,
-      Rcpp::IntegerVector& cols
+      Rcpp::IntegerVector& cols,
+      std::string xyzm
   ) {
     Rcpp::NumericVector nv = sfheaders::shapes::get_vec( nm, cols );
-    return sfg_point( nv );
+    return sfg_point( nv, xyzm );
   }
 
   inline SEXP sfg_point(
       Rcpp::NumericMatrix& nm,
-      Rcpp::StringVector& cols
+      Rcpp::StringVector& cols,
+      std::string xyzm
   ) {
     Rcpp::NumericVector nv = sfheaders::shapes::get_vec( nm, cols );
-    return sfg_point( nv );
+    return sfg_point( nv, xyzm );
   }
 
   // expects only lon/lat/z/m columns in correct order
   inline SEXP sfg_point(
-      Rcpp::DataFrame& df
+      Rcpp::DataFrame& df,
+      std::string xyzm
   ) {
     // expecting single-row data.frame
     Rcpp::NumericVector nv = sfheaders::shapes::get_vec( df );
-    return sfg_point( nv );
+    return sfg_point( nv, xyzm );
   }
 
   inline SEXP sfg_point(
     Rcpp::DataFrame& df,
-    Rcpp::IntegerVector& cols
+    Rcpp::IntegerVector& cols,
+    std::string xyzm
   ) {
     //Rcpp::NumericMatrix nm = sfheaders::utils::df_sfg_matrix( df );
     Rcpp::NumericVector nv = sfheaders::shapes::get_vec( df, cols );
-    return sfg_point( nv );
+    return sfg_point( nv, xyzm );
   }
 
   inline SEXP sfg_point(
     Rcpp::DataFrame& df,
-    Rcpp::StringVector& cols
+    Rcpp::StringVector& cols,
+    std::string xyzm
   ) {
     Rcpp::NumericVector nv = sfheaders::shapes::get_vec( df, cols );
-    return sfg_point( nv );
+    return sfg_point( nv, xyzm );
   }
 
   inline SEXP sfg_point(
-    SEXP& x
+    SEXP& x,
+    std::string xyzm
   ) {
 
     SEXP xc = Rcpp::clone( x );
@@ -115,25 +125,25 @@ namespace sfg {
     case INTSXP: {
       if( Rf_isMatrix( xc ) ) {
       Rcpp::IntegerMatrix im = Rcpp::as< Rcpp::IntegerMatrix >( xc );
-      return sfg_point( im );
+      return sfg_point( im, xyzm );
     } else {
       Rcpp::IntegerVector iv = Rcpp::as< Rcpp::IntegerVector >( xc );
-      return sfg_point( iv );
+      return sfg_point( iv, xyzm );
     }
     }
     case REALSXP: {
       if( Rf_isMatrix( xc ) ) {
       Rcpp::NumericMatrix nm = Rcpp::as< Rcpp::NumericMatrix >( xc );
-      return sfg_point( nm );
+      return sfg_point( nm, xyzm );
     } else {
       Rcpp::NumericVector nv = Rcpp::as< Rcpp::NumericVector >( xc );
-      return sfg_point( nv );
+      return sfg_point( nv, xyzm );
     }
     }
     case VECSXP: { // data.frame && list?
     if( Rf_inherits( xc, "data.frame") ) {
       Rcpp::DataFrame df = Rcpp::as< Rcpp::DataFrame >( xc );
-      return sfg_point( df );
+      return sfg_point( df, xyzm );
     } // else default
     }
     default: {
@@ -146,7 +156,8 @@ namespace sfg {
 
   inline SEXP sfg_point(
       SEXP& x,
-      Rcpp::IntegerVector& cols
+      Rcpp::IntegerVector& cols,
+      std::string xyzm
   ) {
 
     SEXP xc = Rcpp::clone( x );
@@ -155,25 +166,25 @@ namespace sfg {
     case INTSXP: {
       if( Rf_isMatrix( xc ) ) {
         Rcpp::IntegerMatrix im = Rcpp::as< Rcpp::IntegerMatrix >( xc );
-        return sfg_point( im, cols );
+        return sfg_point( im, cols, xyzm );
     } else {
       Rcpp::IntegerVector iv = Rcpp::as< Rcpp::IntegerVector >( xc );
-      return sfg_point( iv, cols );
+      return sfg_point( iv, cols, xyzm );
     }
     }
     case REALSXP: {
       if( Rf_isMatrix( xc ) ) {
       Rcpp::NumericMatrix nm = Rcpp::as< Rcpp::NumericMatrix >( xc );
-      return sfg_point( nm, cols );
+      return sfg_point( nm, cols, xyzm );
     } else {
       Rcpp::NumericVector nv = Rcpp::as< Rcpp::NumericVector >( xc );
-      return sfg_point( nv, cols );
+      return sfg_point( nv, cols, xyzm );
     }
     }
     case VECSXP: {
       if ( Rf_inherits( x, "data.frame" ) ) {
         Rcpp::DataFrame df = Rcpp::as< Rcpp::DataFrame >( xc );
-        return sfg_point( df, cols );
+        return sfg_point( df, cols, xyzm );
       } //else - default
     }
     default: {
@@ -185,12 +196,13 @@ namespace sfg {
 
   inline SEXP sfg_point(
       SEXP& x,
-      Rcpp::StringVector& cols
+      Rcpp::StringVector& cols,
+      std::string xyzm
   ) {
     // with string columns it must be a data.frame(?)
 
     Rcpp::DataFrame df = Rcpp::as< Rcpp::DataFrame >( x );
-    return sfg_point( df, cols );
+    return sfg_point( df, cols, xyzm );
   }
 
 
@@ -200,21 +212,24 @@ namespace sfg {
    */
   inline SEXP sfg_point(
     SEXP& x,
-    SEXP& cols
+    SEXP& cols,
+    std::string xyzm      // this _can_ come from R, but it can also be 'empty' std::string("") (default string constructor)
+    // iff it's empty, it means it has to be inferred.
+    //
   ) {
 
     if( Rf_isNull( cols ) ) {
-      return sfg_point( x );
+      return sfg_point( x, xyzm );
     }
     switch( TYPEOF( cols ) ) {
     case REALSXP: {}
     case INTSXP: {
       Rcpp::IntegerVector int_cols = Rcpp::as< Rcpp::IntegerVector >( cols );
-      return sfg_point( x, int_cols );
+      return sfg_point( x, int_cols, xyzm );
     }
     case STRSXP: {
       Rcpp::StringVector str_cols = Rcpp::as< Rcpp::StringVector >( cols );
-      return sfg_point( x, str_cols );
+      return sfg_point( x, str_cols, xyzm );
     }
     default: {
       Rcpp::stop("sfheaders - unknown column types");  // #nocov
