@@ -13,32 +13,35 @@ namespace sfg {
   // polygon is a list of linestrings (matrices)
   inline SEXP sfg_polygon(
       Rcpp::IntegerMatrix& im,
+      std::string xyzm,
       bool close = true
   ) {
     Rcpp::List mls( 1 );
     mls[0] = sfheaders::polygon_utils::close_polygon( im, close );
     R_xlen_t n_col = im.ncol();
 
-    sfheaders::sfg::make_sfg( mls, n_col, sfheaders::sfg::SFG_POLYGON );
+    sfheaders::sfg::make_sfg( mls, n_col, sfheaders::sfg::SFG_POLYGON, xyzm );
 
     return mls;
   }
 
   inline SEXP sfg_polygon(
       Rcpp::NumericMatrix& nm,
+      std::string xyzm,
       bool close = true
   ) {
     Rcpp::List mls( 1 );
     mls[0] = sfheaders::polygon_utils::close_polygon( nm, close );;
     R_xlen_t n_col = nm.ncol();
 
-    sfheaders::sfg::make_sfg( mls, n_col, sfheaders::sfg::SFG_POLYGON );
+    sfheaders::sfg::make_sfg( mls, n_col, sfheaders::sfg::SFG_POLYGON, xyzm );
 
     return mls;
   }
 
   inline SEXP sfg_polygon(
       Rcpp::DataFrame& df,
+      std::string xyzm,
       bool close = true
   ) {
     Rcpp::List mls( 1 );
@@ -46,18 +49,19 @@ namespace sfg {
     mls[0] = sfheaders::polygon_utils::close_polygon( nm, close );;
     R_xlen_t n_col = nm.ncol();
 
-    sfheaders::sfg::make_sfg( mls, n_col, sfheaders::sfg::SFG_POLYGON );
+    sfheaders::sfg::make_sfg( mls, n_col, sfheaders::sfg::SFG_POLYGON, xyzm );
 
     return mls;
   }
 
   inline SEXP sfg_polygon(
       Rcpp::List& lst,
+      std::string xyzm,
       bool close = true
   ) {
 
     lst = sfheaders::polygon_utils::close_polygon( lst, close );
-    sfheaders::sfg::make_sfg( lst, sfheaders::sfg::SFG_POLYGON );
+    sfheaders::sfg::make_sfg( lst, sfheaders::sfg::SFG_POLYGON, xyzm );
 
     return lst;
   }
@@ -66,55 +70,61 @@ namespace sfg {
   inline SEXP sfg_polygon(
       Rcpp::DataFrame& df,
       Rcpp::IntegerVector& cols,
+      std::string xyzm,
       bool close = true
   ) {
     Rcpp::NumericMatrix nm = sfheaders::shapes::get_mat( df, cols );
-    return sfg_polygon( nm, close );
+    return sfg_polygon( nm, xyzm, close );
   }
 
   inline SEXP sfg_polygon(
       Rcpp::DataFrame& df,
       Rcpp::StringVector& cols,
+      std::string xyzm,
       bool close = true
   ) {
     Rcpp::NumericMatrix nm = sfheaders::shapes::get_mat( df, cols );
-    return sfg_polygon( nm, close );
+    return sfg_polygon( nm, xyzm, close );
   }
 
   inline SEXP sfg_polygon(
       Rcpp::NumericMatrix& nm,
       Rcpp::IntegerVector& cols,
+      std::string xyzm,
       bool close = true
   ) {
     Rcpp::NumericMatrix nm2 = sfheaders::shapes::get_mat( nm, cols );
-    return sfg_polygon( nm2, close );
+    return sfg_polygon( nm2, xyzm, close );
   }
 
   inline SEXP sfg_polygon(
       Rcpp::NumericMatrix& nm,
       Rcpp::StringVector& cols,
+      std::string xyzm,
       bool close = true
   ) {
     Rcpp::NumericMatrix nm2 = sfheaders::shapes::get_mat( nm, cols );
-    return sfg_polygon( nm2, close );
+    return sfg_polygon( nm2, xyzm, close );
   }
 
   inline SEXP sfg_polygon(
       Rcpp::IntegerMatrix& im,
       Rcpp::IntegerVector& cols,
+      std::string xyzm,
       bool close = true
   ) {
     Rcpp::IntegerMatrix im2 = sfheaders::shapes::get_mat( im, cols );
-    return sfg_polygon( im2, close );
+    return sfg_polygon( im2, xyzm, close );
   }
 
   inline SEXP sfg_polygon(
       Rcpp::IntegerMatrix& im,
       Rcpp::StringVector& cols,
+      std::string xyzm,
       bool close = true
   ) {
     Rcpp::IntegerMatrix im2 = sfheaders::shapes::get_mat( im, cols );
-    return sfg_polygon( im2, close );
+    return sfg_polygon( im2, xyzm, close );
   }
 
   // We're still on single sfg objects.
@@ -123,65 +133,72 @@ namespace sfg {
       Rcpp::DataFrame& df,
       Rcpp::IntegerVector& cols,
       int& id_col,
+      std::string xyzm,
       bool close = true
   ) {
     Rcpp::List lst = sfheaders::shapes::get_listMat( df, cols, id_col );
-    return sfg_polygon( lst, close );
+    return sfg_polygon( lst, xyzm, close );
   }
 
   inline SEXP sfg_polygon(
       Rcpp::DataFrame& df,
       Rcpp::StringVector& cols,
       Rcpp::String& id_col,
+      std::string xyzm,
       bool close = true
   ) {
     Rcpp::List lst = sfheaders::shapes::get_listMat( df, cols, id_col );
-    return sfg_polygon( lst, close );
+    return sfg_polygon( lst, xyzm, close );
   }
 
   inline SEXP sfg_polygon(
       Rcpp::NumericMatrix& nm,
       Rcpp::IntegerVector& cols,
       int& id_col,
+      std::string xyzm,
       bool close = true
   ) {
     Rcpp::List lst = sfheaders::shapes::get_listMat( nm, cols, id_col );
-    return sfg_polygon( lst, close );
+    return sfg_polygon( lst, xyzm, close );
   }
 
   inline SEXP sfg_polygon(
       Rcpp::NumericMatrix& nm,
       Rcpp::StringVector& cols,
       Rcpp::String& id_col,
+      std::string xyzm,
       bool close = true
   ) {
     Rcpp::List lst = sfheaders::shapes::get_listMat( nm, cols, id_col );
-    return sfg_polygon( lst, close );
+    return sfg_polygon( lst, xyzm, close );
   }
 
   inline SEXP sfg_polygon(
       Rcpp::IntegerMatrix& im,
       Rcpp::IntegerVector& cols,
       int& id_col,
+      std::string xyzm,
       bool close = true
   ) {
     Rcpp::List lst = sfheaders::shapes::get_listMat( im, cols, id_col );
-    return sfg_polygon( lst, close );
+    return sfg_polygon( lst, xyzm, close );
   }
 
   inline SEXP sfg_polygon(
       Rcpp::IntegerMatrix& im,
       Rcpp::StringVector& cols,
       Rcpp::String& id_col,
+      std::string xyzm,
       bool close = true
   ) {
     Rcpp::List lst = sfheaders::shapes::get_listMat( im, cols, id_col );
-    return sfg_polygon( lst, close );
+    return sfg_polygon( lst, xyzm, close );
   }
 
   inline SEXP sfg_polygon(
       SEXP& x,
       Rcpp::IntegerVector& cols,
+      std::string xyzm,
       bool close = true
   ) {
     switch( TYPEOF( x ) ) {
@@ -190,20 +207,20 @@ namespace sfg {
       Rcpp::stop("sfheaders - expecting a matrix");
     }
       Rcpp::IntegerMatrix im = Rcpp::as< Rcpp::IntegerMatrix >( x );
-      return sfg_polygon( im, cols, close );
+      return sfg_polygon( im, cols, xyzm, close );
     }
     case REALSXP: {
       if( !Rf_isMatrix( x ) ) {
       Rcpp::stop("sfheaders - expecting a matrix");
     }
       Rcpp::NumericMatrix nm = Rcpp::as< Rcpp::NumericMatrix >( x );
-      return sfg_polygon( nm, cols, close );
+      return sfg_polygon( nm, cols, xyzm, close );
     }
     case VECSXP: {
       // #nocov start
       if( Rf_inherits( x, "data.frame") ) {
       Rcpp::DataFrame df = Rcpp::as< Rcpp::DataFrame >( x );
-      return sfg_polygon( df, cols, close );
+      return sfg_polygon( df, cols, xyzm, close);
       // #nocov end
     } // else default
     }
@@ -221,6 +238,7 @@ namespace sfg {
       SEXP& x,
       Rcpp::IntegerVector& cols,
       int& line_id,
+      std::string xyzm,
       bool close = true
   ) {
     switch( TYPEOF( x ) ) {
@@ -229,19 +247,19 @@ namespace sfg {
       Rcpp::stop("sfheaders - expecting a matrix");
     }
       Rcpp::IntegerMatrix im = Rcpp::as< Rcpp::IntegerMatrix >( x );
-      return sfg_polygon( im, cols, line_id, close );
+      return sfg_polygon( im, cols, line_id, xyzm, close);
     }
     case REALSXP: {
       if( !Rf_isMatrix( x ) ) {
       Rcpp::stop("sfheaders - expecting a matrix");
     }
       Rcpp::NumericMatrix nm = Rcpp::as< Rcpp::NumericMatrix >( x );
-      return sfg_polygon( nm, cols, line_id, close );
+      return sfg_polygon( nm, cols, line_id, xyzm, close);
     }
     case VECSXP: {
       if( Rf_inherits( x, "data.frame") ) {
       Rcpp::DataFrame df = Rcpp::as< Rcpp::DataFrame >( x );
-      return sfg_polygon( df, cols, line_id, close );
+      return sfg_polygon( df, cols, line_id, xyzm, close);
     } // else default
     }
     default: {
@@ -291,6 +309,7 @@ namespace sfg {
   inline SEXP sfg_polygon(
       SEXP& x,
       Rcpp::StringVector& cols,
+      std::string xyzm,
       bool close = true
   ) {
     switch( TYPEOF( x ) ) {
@@ -299,19 +318,19 @@ namespace sfg {
       Rcpp::stop("sfheaders - expecting a matrix");
     }
       Rcpp::IntegerMatrix im = Rcpp::as< Rcpp::IntegerMatrix >( x );
-      return sfg_polygon( im, cols, close );
+      return sfg_polygon( im, cols, xyzm, close);
     }
     case REALSXP: {
       if( !Rf_isMatrix( x ) ) {
       Rcpp::stop("sfheaders - expecting a matrix");
     }
       Rcpp::NumericMatrix nm = Rcpp::as< Rcpp::NumericMatrix >( x );
-      return sfg_polygon( nm, cols, close );
+      return sfg_polygon( nm, cols, xyzm, close);
     }
     case VECSXP: {
       if( Rf_inherits( x, "data.frame") ) {
       Rcpp::DataFrame df = Rcpp::as< Rcpp::DataFrame >( x );
-      return sfg_polygon( df, cols, close );
+      return sfg_polygon( df, cols, xyzm, close);
     } // else default
     }
     default: {
@@ -360,6 +379,7 @@ namespace sfg {
       SEXP& x,
       Rcpp::StringVector& cols,
       Rcpp::String& line_id,
+      std::string xyzm,
       bool close = true
   ) {
     switch( TYPEOF( x ) ) {
@@ -368,19 +388,19 @@ namespace sfg {
       Rcpp::stop("sfheaders - expecting a matrix");
     }
       Rcpp::IntegerMatrix im = Rcpp::as< Rcpp::IntegerMatrix >( x );
-      return sfg_polygon( im, cols, line_id, close );
+      return sfg_polygon( im, cols, line_id, xyzm, close);
     }
     case REALSXP: {
       if( !Rf_isMatrix( x ) ) {
       Rcpp::stop("sfheaders - expecting a matrix");
     }
       Rcpp::NumericMatrix nm = Rcpp::as< Rcpp::NumericMatrix >( x );
-      return sfg_polygon( nm, cols, line_id, close );
+      return sfg_polygon( nm, cols, line_id, xyzm, close);
     }
     case VECSXP: {
       if( Rf_inherits( x, "data.frame") ) {
       Rcpp::DataFrame df = Rcpp::as< Rcpp::DataFrame >( x );
-      return sfg_polygon( df, cols, line_id, close );
+      return sfg_polygon( df, cols, line_id, xyzm, close);
     } // else default
     }
     default: {
@@ -394,6 +414,7 @@ namespace sfg {
 
   inline SEXP sfg_polygon(
       SEXP& x,
+      std::string xyzm,
       bool close = true
   ) {
     switch ( TYPEOF( x ) ) {
@@ -402,22 +423,22 @@ namespace sfg {
       Rcpp::stop("sfheaders - expecting a matrix");
     }
       Rcpp::IntegerMatrix im = Rcpp::as< Rcpp::IntegerMatrix >( x );
-      return sfg_polygon( im, close );
+      return sfg_polygon( im, xyzm, close);
     }
     case REALSXP: {
       if( !Rf_isMatrix( x ) ) {
       Rcpp::stop("sfheaders - expecting a matrix");
     }
       Rcpp::NumericMatrix nm = Rcpp::as< Rcpp::NumericMatrix >( x );
-      return sfg_polygon( nm, close );
+      return sfg_polygon( nm, xyzm, close);
     }
     case VECSXP: {
       if( Rf_inherits( x, "data.frame") ) {
       Rcpp::DataFrame df = Rcpp::as< Rcpp::DataFrame >( x );
-      return sfg_polygon( df, close );
+      return sfg_polygon( df, xyzm, close);
     } else {
       Rcpp::List lst = Rcpp::as< Rcpp::List >( x ); // #nocov
-      return sfg_polygon( lst, close );                    // #nocov
+      return sfg_polygon( lst, xyzm, close);                    // #nocov
     }
     }
     default: {
@@ -431,20 +452,21 @@ namespace sfg {
   inline SEXP sfg_polygon(
       SEXP& x,
       SEXP& cols,
+      std::string xyzm,
       bool close = true
   ) {
     if( Rf_isNull( cols ) ) {
-      return sfg_polygon( x, close );
+      return sfg_polygon( x, xyzm, close);
     }
     switch( TYPEOF( cols ) ) {
     case REALSXP: {}
     case INTSXP: {
       Rcpp::IntegerVector iv = Rcpp::as< Rcpp::IntegerVector >( cols );
-      return sfg_polygon( x, iv, close );
+      return sfg_polygon( x, iv, xyzm, close);
     }
     case STRSXP: {
       Rcpp::StringVector sv = Rcpp::as< Rcpp::StringVector >( cols );
-      return sfg_polygon( x, sv, close );
+      return sfg_polygon( x, sv, xyzm, close);
     }
     default: {
       Rcpp::stop("sfheaders - unknown column types");  // #nocov
@@ -458,6 +480,7 @@ namespace sfg {
       SEXP& x,
       SEXP& cols,
       Rcpp::String& line_id,
+      std::string xyzm,
       bool close = true
   ) {
     if( Rf_isNull( cols ) ) {
@@ -465,7 +488,7 @@ namespace sfg {
       Rcpp::StringVector id_cols( 1 );
       id_cols[0] = line_id;
       SEXP other_cols = sfheaders::utils::other_columns( x, id_cols );
-      return sfg_polygon( x, other_cols, line_id, close );
+      return sfg_polygon( x, other_cols, line_id, xyzm, close);
       // #nocov end
     }
     switch( TYPEOF( cols ) ) {
@@ -476,7 +499,7 @@ namespace sfg {
     // }
     case STRSXP: {
       Rcpp::StringVector sv = Rcpp::as< Rcpp::StringVector >( cols );
-      return sfg_polygon( x, sv, line_id, close );
+      return sfg_polygon( x, sv, line_id, xyzm, close);
     }
     default: {
       Rcpp::stop("sfheaders - unknown column types"); // #nocov
@@ -490,6 +513,7 @@ namespace sfg {
       SEXP& x,
       SEXP& cols,
       int& line_id,
+      std::string xyzm,
       bool close = true
   ) {
 
@@ -498,14 +522,14 @@ namespace sfg {
       Rcpp::IntegerVector id_cols( 1 );
       id_cols[0] = line_id;
       SEXP other_cols = sfheaders::utils::other_columns( x, id_cols );
-      return sfg_polygon( x, other_cols, line_id, close );
+      return sfg_polygon( x, other_cols, line_id, xyzm, close);
       // #nocov end
     }
     switch( TYPEOF( cols ) ) {
     case REALSXP: {}
     case INTSXP: {
       Rcpp::IntegerVector iv = Rcpp::as< Rcpp::IntegerVector >( cols );
-      return sfg_polygon( x, iv, line_id, close );
+      return sfg_polygon( x, iv, line_id, xyzm, close);
     }
       // case STRSXP: {
       //   Rcpp::StringVector sv = Rcpp::as< Rcpp::StringVector >( cols );
@@ -522,22 +546,23 @@ namespace sfg {
       SEXP& x,
       SEXP& cols,
       SEXP& line_id,
+      std::string xyzm,
       bool close = true
   ) {
     if( Rf_isNull( line_id ) ) {
-      return sfg_polygon( x, cols, close );
+      return sfg_polygon( x, cols, xyzm, close);
     }
     switch( TYPEOF( line_id ) ) {
     case REALSXP: {}
     case INTSXP: {
       Rcpp::IntegerVector iv = Rcpp::as< Rcpp::IntegerVector >( line_id );
       int i = iv[0];
-      return sfg_polygon( x, cols, i, close );
+      return sfg_polygon( x, cols, i, xyzm, close);
     }
     case STRSXP: {
       Rcpp::StringVector sv = Rcpp::as< Rcpp::StringVector >( line_id );
       Rcpp::String s = sv[0];
-      return sfg_polygon( x, cols, s, close );
+      return sfg_polygon( x, cols, s, xyzm, close);
     }
     default: {
       Rcpp::stop("sfheaders - unknown column types");  // #nocov
@@ -548,14 +573,44 @@ namespace sfg {
 
   inline SEXP remove_polygon_holes(
       Rcpp::List& sfg_poly,
+      std::string xyzm,
       bool close = true
   ){
     Rcpp::List res(1);
     res[0] = sfg_poly[0];
-    return sfg_polygon( res, close );
+    return sfg_polygon( res, xyzm, close);
   }
 
 
+  // sfg_box will convert a bounding-box 4-value vector into a polygon
+  template< int RTYPE >
+  inline SEXP sfg_box( Rcpp::Vector< RTYPE > x ) {
+    if ( x.length() != 4 ) {
+      Rcpp::stop("sfheaders - box requires a 4-value vector");
+    }
+
+    Rcpp::Matrix< RTYPE > mat( 5, 2 );
+    Rcpp::IntegerVector bl = {0,1};
+    Rcpp::IntegerVector br = {2,1};
+    Rcpp::IntegerVector tr = {2,3};
+    Rcpp::IntegerVector tl = {0,3};
+
+    Rcpp::Vector< RTYPE > blv = x[ bl ];
+    Rcpp::Vector< RTYPE > brv = x[ br ];
+    Rcpp::Vector< RTYPE > trv = x[ tr ];
+    Rcpp::Vector< RTYPE > tlv = x[ tl ];
+
+    mat( 0, Rcpp::_ ) = blv;
+    mat( 1, Rcpp::_ ) = brv;
+    mat( 2, Rcpp::_ ) = trv;
+    mat( 3, Rcpp::_ ) = tlv;
+    mat( 4, Rcpp::_ ) = blv;
+
+    std::string xyzm = "XY";
+    bool close = false;
+    return sfg_polygon( mat, xyzm, close );
+  }
+
 } // sfg
 } // sfheaders
 
