@@ -9,115 +9,115 @@ test_that("various objects converted to sfc_LINESTRING",{
   }
 
   v <- 1:3
-  res <- sfheaders:::rcpp_sfc_linestring(v, NULL, NULL)
+  res <- sfheaders:::rcpp_sfc_linestring(v, NULL, NULL, "")
   expect_equal( attr( res, "class" ), c("sfc_LINESTRING", "sfc") )
   expect_true( is_linestring( res ) )
 
   v <- c(1.2,3)
-  res <- sfheaders:::rcpp_sfc_linestring(v, NULL, NULL)
+  res <- sfheaders:::rcpp_sfc_linestring(v, NULL, NULL, "")
   expect_equal( attr( res, "class" ), c("sfc_LINESTRING", "sfc") )
   expect_true( is_linestring( res ) )
 
   m <- matrix(1:4, ncol = 2)
   m <- cbind(c(1L,1L), m)
-  res <- sfheaders:::rcpp_sfc_linestring(m, c(1L,2L), 0L)
+  res <- sfheaders:::rcpp_sfc_linestring(m, c(1L,2L), 0L, "")
   expect_equal( attr( res, "class" ), c("sfc_LINESTRING", "sfc") )
   expect_true( is_linestring( res ) )
 
   m <- matrix(1:4, ncol = 2)
-  res <- sfheaders:::rcpp_sfc_linestring(m, NULL, NULL)
+  res <- sfheaders:::rcpp_sfc_linestring(m, NULL, NULL, "")
   expect_equal( attr( res, "class" ), c("sfc_LINESTRING", "sfc") )
   expect_true( is_linestring( res ) )
 
   m <- matrix(c(1.2,3,4,5), ncol = 2)
-  res <- sfheaders:::rcpp_sfc_linestring(m, NULL, NULL)
+  res <- sfheaders:::rcpp_sfc_linestring(m, NULL, NULL, "")
   expect_equal( attr( res, "class" ), c("sfc_LINESTRING", "sfc") )
   expect_true( is_linestring( res ) )
 
   m <- matrix(1:4, ncol = 2)
-  res <- sfheaders:::rcpp_sfc_linestring(m, c(0,1), NULL)
+  res <- sfheaders:::rcpp_sfc_linestring(m, c(0,1), NULL, "")
   expect_equal( attr( res, "class" ), c("sfc_LINESTRING", "sfc") )
   expect_true( is_linestring( res ) )
 
   m <- matrix(c(1.2,3,4,5), ncol = 2)
-  res <- sfheaders:::rcpp_sfc_linestring(m, c(0,1), NULL)
+  res <- sfheaders:::rcpp_sfc_linestring(m, c(0,1), NULL, "")
   expect_equal( attr( res, "class" ), c("sfc_LINESTRING", "sfc") )
   expect_true( is_linestring( res ) )
 
   m <- matrix(1:4, ncol = 2)
   df <- as.data.frame( m )
-  res <- sfheaders:::rcpp_sfc_linestring(df, NULL, NULL)
+  res <- sfheaders:::rcpp_sfc_linestring(df, NULL, NULL, "")
   expect_equal( attr( res, "class" ), c("sfc_LINESTRING", "sfc") )
   expect_true( is_linestring( res ) )
 
   m <- matrix(1:4, ncol = 2)
   df <- as.data.frame( m )
-  res <- sfheaders:::rcpp_sfc_linestring(df, c(0,1), NULL)
+  res <- sfheaders:::rcpp_sfc_linestring(df, c(0,1), NULL, "")
   expect_equal( attr( res, "class" ), c("sfc_LINESTRING", "sfc") )
   expect_true( is_linestring( res ) )
 
   m <- matrix(1:4, ncol = 2)
   df <- as.data.frame( m )
   m <- as.matrix( df )
-  res <- sfheaders:::rcpp_sfc_linestring(df, c("V1","V2"), NULL)
+  res <- sfheaders:::rcpp_sfc_linestring(df, c("V1","V2"), NULL, "")
   expect_equal( attr( res, "class" ), c("sfc_LINESTRING", "sfc") )
   expect_true( is_linestring( res ) )
 
   m <- matrix(c(1.2,3,4,5), ncol = 2)
   df <- as.data.frame( m )
   m <- as.matrix( df )
-  res <- sfheaders:::rcpp_sfc_linestring(df, c("V1","V2"), NULL)
+  res <- sfheaders:::rcpp_sfc_linestring(df, c("V1","V2"), NULL, "")
   expect_equal( attr( res, "class" ), c("sfc_LINESTRING", "sfc") )
   expect_true( is_linestring( res ) )
 
 
   m <- matrix(1:4, ncol = 2)
   df <- as.data.frame( m )
-  res <- sfheaders:::rcpp_sfc_linestring(df, c("V1","V2"), NULL)
+  res <- sfheaders:::rcpp_sfc_linestring(df, c("V1","V2"), NULL, "")
   expect_equal( attr( res, "class" ), c("sfc_LINESTRING", "sfc") )
   expect_true( is_linestring( res ) )
 
   m <- matrix(1:8, ncol = 2)
   m <- cbind(m, c(1,1,2,2))
-  res <- sfheaders:::rcpp_sfc_linestring(m, NULL, 2)
+  res <- sfheaders:::rcpp_sfc_linestring(m, NULL, 2, "")
   expect_true( is_linestring( res ) )
   expect_equal( attr( res, "class" ), c("sfc_LINESTRING", "sfc") )
 
   m <- matrix(1:8, ncol = 2)
   m <- cbind(m, c(1,1,2,2))
-  res <- sfheaders:::rcpp_sfc_linestring(m, c(0,1), 2)
+  res <- sfheaders:::rcpp_sfc_linestring(m, c(0,1), 2, "")
   expect_true( is_linestring( res ) )
   expect_equal( attr( res, "class" ), c("sfc_LINESTRING", "sfc") )
 
   m <- matrix(1:8, ncol = 2)
   m <- cbind(m, c(1,1,2,2))
   df <- as.data.frame( m )
-  res <- sfheaders:::rcpp_sfc_linestring(df, NULL, 2)
+  res <- sfheaders:::rcpp_sfc_linestring(df, NULL, 2, "")
   expect_true( is_linestring( res ) )
   expect_equal( attr( res, "class" ), c("sfc_LINESTRING", "sfc") )
 
   m <- matrix(1:8, ncol = 2)
   m <- cbind(m, c(1L,1L,2L,2L))
-  res <- sfheaders:::rcpp_sfc_linestring(m, c(0L,1L), 2L)
+  res <- sfheaders:::rcpp_sfc_linestring(m, c(0L,1L), 2L,"")
   expect_true( is_linestring( res ) )
   expect_equal( attr( res, "class" ), c("sfc_LINESTRING", "sfc") )
 
   m <- matrix(1:8, ncol = 2)
   m <- cbind(m, c(1,1,2,2))
-  res <- sfheaders:::rcpp_sfc_linestring(m, c(0L,1L), 2L)
+  res <- sfheaders:::rcpp_sfc_linestring(m, c(0L,1L), 2L,"")
   expect_true( is_linestring( res ) )
   expect_equal( attr( res, "class" ), c("sfc_LINESTRING", "sfc") )
 
   m <- matrix(1:2, ncol = 2)
   m <- cbind(m, c(1))
-  res <- sfheaders:::rcpp_sfc_linestring(m, c(0L,1L), 2L)
+  res <- sfheaders:::rcpp_sfc_linestring(m, c(0L,1L), 2L,"")
   expect_true( is_linestring( res ) )
   expect_equal( attr( res, "class" ), c("sfc_LINESTRING", "sfc") )
 
   m <- matrix(1:8, ncol = 2)
   m <- cbind(m, c(1,1,2,2))
   df <- as.data.frame( m )
-  res <- sfheaders:::rcpp_sfc_linestring(df, c(0,1), 2)
+  res <- sfheaders:::rcpp_sfc_linestring(df, c(0,1), 2,"")
   expect_true( is_linestring( res ) )
   expect_equal( attr( res, "class" ), c("sfc_LINESTRING", "sfc") )
 
@@ -125,7 +125,7 @@ test_that("various objects converted to sfc_LINESTRING",{
   m <- cbind(m, c(1L,1L,2L,2L) )
   df <- as.data.frame( m )
   m <- as.matrix( df )
-  res <- sfheaders:::rcpp_sfc_linestring(m, c("V1","V2"), NULL )
+  res <- sfheaders:::rcpp_sfc_linestring(m, c("V1","V2"), NULL, "")
   expect_true( is_linestring( res ) )
   expect_equal( attr( res, "class" ), c("sfc_LINESTRING", "sfc") )
 
@@ -133,14 +133,14 @@ test_that("various objects converted to sfc_LINESTRING",{
   m <- cbind(m, c(1,1,2,2) )
   df <- as.data.frame( m )
   m <- as.matrix( df )
-  res <- sfheaders:::rcpp_sfc_linestring(m, c("V1","V2"), NULL )
+  res <- sfheaders:::rcpp_sfc_linestring(m, c("V1","V2"), NULL, "")
   expect_true( is_linestring( res ) )
   expect_equal( attr( res, "class" ), c("sfc_LINESTRING", "sfc") )
 
   m <- matrix(c(1.2,2:8), ncol = 2)
   m <- cbind(m, c(1,1,2,2))
   df <- as.data.frame( m )
-  res <- sfheaders:::rcpp_sfc_linestring(df, c("V1","V2"), NULL )
+  res <- sfheaders:::rcpp_sfc_linestring(df, c("V1","V2"), NULL, "")
   expect_true( is_linestring( res ) )
   expect_equal( attr( res, "class" ), c("sfc_LINESTRING", "sfc") )
 
@@ -148,7 +148,7 @@ test_that("various objects converted to sfc_LINESTRING",{
   m <- cbind(c(1L,1L), m)
   df <- as.data.frame( m )
   m <- as.matrix( df )
-  res <- sfheaders:::rcpp_sfc_linestring(m, c("V1","V2"), "V3")
+  res <- sfheaders:::rcpp_sfc_linestring(m, c("V1","V2"), "V3", "")
   expect_equal( attr( res, "class" ), c("sfc_LINESTRING", "sfc") )
   expect_true( is_linestring( res ) )
 
@@ -156,30 +156,29 @@ test_that("various objects converted to sfc_LINESTRING",{
   m <- cbind(m, c(1,1,2,2) )
   df <- as.data.frame( m )
   m <- as.matrix( df )
-  res <- sfheaders:::rcpp_sfc_linestring(m, c("V1","V2"), "V3" )
+  res <- sfheaders:::rcpp_sfc_linestring(m, c("V1","V2"), "V3", "")
   expect_true( is_linestring( res ) )
   expect_equal( attr( res, "class" ), c("sfc_LINESTRING", "sfc") )
 
   m <- matrix(c(1.2,2:8), ncol = 2)
   m <- cbind(m, c(1,1,2,2))
   df <- as.data.frame( m )
-  res <- sfheaders:::rcpp_sfc_linestring(df, c("V1","V2"), "V3" )
+  res <- sfheaders:::rcpp_sfc_linestring(df, c("V1","V2"), "V3", "")
   expect_true( is_linestring( res ) )
   expect_equal( attr( res, "class" ), c("sfc_LINESTRING", "sfc") )
 
 
-
   m <- matrix(1:8, ncol = 2)
   m <- cbind(m, c(1,1,2,2))
   df <- as.data.frame( m )
-  res <- sfheaders:::rcpp_sfc_linestring(df, c("V1","V2"), c("V3"))
+  res <- sfheaders:::rcpp_sfc_linestring(df, c("V1","V2"), c("V3"), "")
   expect_true( is_linestring( res ) )
   expect_equal( attr( res, "class" ), c("sfc_LINESTRING", "sfc") )
 
   m <- matrix(1:2, ncol = 2)
   m <- cbind(m, c(1))
   df <- as.data.frame( m )
-  res <- sfheaders:::rcpp_sfc_linestring(df, c("V1","V2"), c("V3"))
+  res <- sfheaders:::rcpp_sfc_linestring(df, c("V1","V2"), c("V3"), "")
   expect_true( is_linestring( res ) )
   expect_equal( attr( res, "class" ), c("sfc_LINESTRING", "sfc") )
 
@@ -187,12 +186,12 @@ test_that("various objects converted to sfc_LINESTRING",{
   m <- matrix(1:2, ncol = 2)
   m <- cbind(m, c(1))
   df <- as.data.frame( m )
-  res <- sfheaders:::rcpp_sfc_linestring(df, c(0L,1L), 2L )
+  res <- sfheaders:::rcpp_sfc_linestring(df, c(0L,1L), 2L, "")
   expect_true( is_linestring( res ) )
   expect_equal( attr( res, "class" ), c("sfc_LINESTRING", "sfc") )
 
   m <- matrix(1:4, ncol = 2)
-  expect_error( sfheaders:::rcpp_sfc_linestring(m, NULL, 0), "sfheaders - incorrect number of geometry columns")
+  expect_error( sfheaders:::rcpp_sfc_linestring(m, NULL, 0, ""), "sfheaders - incorrect number of geometry columns")
 
 })
 
@@ -233,7 +232,7 @@ test_that("vectorised version works",{
   m1 <- matrix(1:12, ncol = 3)
   m2 <- matrix(1:12, ncol = 3)
   lst <- list( m1, m2 )
-  res <- sfheaders:::rcpp_sfc_linestrings( lst )
+  res <- sfheaders:::rcpp_sfc_linestrings( lst, "" )
   expect_true( all( sapply( res, is_linestring ) ) )
 
 })
